/*--------------------------------------------------------------------------
 *  Copyright (c) 2009-2012 Igor Daniloff.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found in this directory.
 *------------------------------------------------------------------------*/

#ifndef DW_RS_SDK_H_
#define DW_RS_SDK_H_

#include "DwRsSDK_Log.H"
#include "DwRsSDK_Mail.H"
#include "DwRsSDK_IProcessingInfo.H"
#include "DwRsSDK_IReceiver.H"
#include "DwRsSDK_ISender.H"
#include "DwRsSDK_ICallback.H"
#include "DwRsSDK_Ptr.H"

/**\file
   \~russian Подключение всех заголовков Dr.Web (R) Receiver/Sender API
   \~english Connection of all Dr.Web (R) Receiver/Sender API headers
   \~
 */

/** \~russian \mainpage Документация по Dr.Web (R) Receiver/Sender API
\~english \mainpage Documentation on Dr.Web (R) Receiver /Sender API

\~russian

Данный API предоставляет инструментарий для разработки дополнительных модулей
компонентов Receiver и Sender программного комплекса Dr.Web (R) MailD на
 языках C и C++. Потребность в дополнительных модулях компонентов Receiver и
Sender может возникнуть, например, при необходимости добавления в Dr.Web MailD
поддержки новых почтовых систем.

Компоненты Receiver и Sender должны быть запущены на том же компьютере,
что и весь Dr.Web MailD, так как при взаимодействии между компонентами
программного комплекса используется файловая система.

Для компиляции компонентов Receiver и Sender требуется линковка с библиотекой
из дистрибутива Dr.Web MailD: libdwrssdk.so (примеры см. в Makefile
соответствующих примеров). Так же для работы необходимы библиотеки libdwlog.so
и libboost_thread.so.

Компонент Receiver отвечает за получение почтовых сообщений (напрямую от
почтовых систем, или каким-либо другим способом), и последующую их передачу
модулю drweb-maild на проверку. Модули компонента Receiver могут поддерживать
возможность модификации и отправления полученных сообщений, принимая
результаты проверки писем от модуля drweb-maild.

Компонент Sender отвечает за отправку различных писем из программного
комплекса Dr.Web MailD, как тех, которые не смог отправить компонент Receiver,
так и тех, что были сгенерированы внутри программного комплекса.
Одновременно в составе комплекса может работать несколько компонентов Sender.

Модуль drweb-maild может отдавать результаты проверки конкретного письма в
синхронном и асинхронном режимах. В синхронном режиме модуль drweb-maild
передает письмо плагинам и ожидает окончания его проверки. Получив результаты
проверки, модуль передает их компоненту Receiver, отправляет письмо (если оно
не заблокировано по результатам проверки), и только после этого может забрать
следующее письмо на обработку.
Для того, чтобы модуль drweb-maild мог работать в синхронном режиме, все его
плагины должны быть подключены через очередь BeforeQueueFilters. При таком
способе подключения у плагинов отсутствует возможность откладывать проверку
писем или сохранять их в БД - они должны максимально быстро отдавать
результаты проверки писем.

При работе в асинхронном режиме, модуль drweb-maild, забрав письмо на обработку,
передает его плагинам, затем отправляет компоненту Receiver сообщение о том, что
проверка письма началась, но результатов проверки еще нет.
Сразу после этого, не дожидаясь окончания проверки письма плагинами, модуль
drweb-maild можеть забирать на обработку следующее письмо. В том случае,
если модуль drweb-maild работает в асинхронном режиме, плагины могут подключаться
к нему как через очередь BeforeQueueFilters, так и через очередь AfterQueueFilters.
Плагины, подключенные вторым способом, имеют полный доступ к БД и могут на
некоторое время откладывать проверку писем.

В том случае, если модуль drweb-maild работает в синхронном режиме, обработанные
письма сразу отправляются через компонент Receiver (если он предусматривает
такую возможность). Если же drweb-maild работает в асинхронном режиме, то для
отправки обработанных писем через компонент Receiver используется сервис Callback.
Этот сервис работает следующим образом:
- компонент Receiver открывает прослушивающее соединение и сообщает модулю
drweb-maild, что он поддерживает сервис Callback;
- модуль drweb-maild после завершения обработки письма обращается к сервису
Callback с запросом, может ли компонент Receiver отправить это письмо;
- если такая возможность есть, отправкой письма занимается компонент Receiver,
если же нет - письмо отправляется с помощью компонента Sender.

Поддерживается одновременная работа нескольких модулей компонента Receiver
(с возможностью для каждого из них запускать свой сервис Callback): это
позволяет получать и обрабатывать почту сразу из нескольких источников.

  \~english

This API provides tools for development of additional modules of the Receiver
and the Sender components of Dr.Web (R) MailD in C and C++ programming
languages. Additional modules for the Receiver and the Sender components may
become necessary, when support of some new mail systems in Dr.Web (R) MailD
is required.
The Receiver and the Sender components must be executed on the same computer,
where Dr.Web MailD is running, because the file system is used in interaction of
all these components. To compile the Receiver and the Sender components, a linkage
to three libraries from the Dr.Web MailD distribution package is required: libdwrssdk.so,
libboost_thread.so and libdwlog.so. For examples refer to Makefile. of corresponding
examples.

The Receiver component is responsible for the receipt of mail messages (directly
from mail systems or by any other means) and for their subsequent transfer to the
drweb-maild module for checkup. Modules of the Receiver component support
modification and dispatch of messages according to check results received from
drweb-maild module.

The Sender component is responsible for dispatch of various messages from
Dr.Web MailD (e.g. messages, which the Receiver component has failed to send,
or messages generated by some applications inside the software complex).
Several Sender components can operate within the program complex at a time.

The drweb-maild module can return check results of a specific message both in
asynchronous and synchronous modes. In synchronous mode drweb-maild module
forwards the message to plug-ins and waits for check results. After receipt of these
results, drweb-maild module passes them to the Receiver component, sends the
message (if it is not blocked according to check results), and only after that it can
take the next message for processing. To make drweb-maild module operate in
synchronous mode, all its plug-ins must be connected via the BeforeQueueFilters
event queue. Plug-ins from this queue cannot postpone message check or put
messages to the database, because they must return check results as fast as
possible.

In asynchronous mode drweb-maild module takes a message for processing,
then passes it to plug-ins and after that sends a Receiver component a
notification, that message check has already begun, but its results are not
ready yet.
Immediately after that, without waiting for plug-ins to complete message check,
drweb-maild module can take the next message for processing. In this case, if
drweb-maild module operates in an asynchronous mode, plug-ins can connect to
it via the BeforeQueueFilters event queue and via the AfterQueueFilters event
queue. Plug-ins from AfterQueueFilters event queue usually have full access to
the database and therefore can postpone message check.

If drweb-maild module operates in synchronous mode, all processed messages are
immediately dispatched via the Receiver component (if it provides such a possibility).
If drweb-maild module operates in asynchronous mode, the Callback service is used
to send processed messages via the Receiver component. This service works as follows:
- the Receiver component opens a listening connection and informs drweb-maild
module that it supports the Callback service;
- after message processing is completed, drweb-maild module refers to the Callback
service with a request, whether the Receiver component can send this message or not;
- If there exists such a possibility, the message is sent by the Receiver component,
otherwise the message is sent by the Sender component.

Simultaneous operation of several modules of the Receiver component is supported
(each of them can use its own Callback service). It allows to receive and analyze mail
from several sources simultaneously.

 \~
 - \ref receiver
 \n
 - \ref sender

\~russian \section cont Контакты
\~english \section cont Contacts

\~russian
По всем вопросам, связанным с лицензированием модулей, разработанных с помощью
данного SDK, обращайтесь по адресу: <a href="mailto:partners@drweb.com">partners@drweb.com</a>.

\~english
Please contact us regarding all issues connected with licensing of modules developed
with the help of this SDK: <a href="mailto:partners@drweb.com">partners@drweb.com</a>.


*/
/**
   \~russian \page receiver Разработка компонента Receiver
   \~english \page receiver Development of the Receiver component

   \~russian
Процесс выполнения модуля компонента Receiver можно разделить на несколько этапов:

- Инициализация лога DwRsSDK.
- Инициализация компонента Receiver.
- (*) Инициализация сервиса Callback и регистрация функции обработчика.

Далее, для каждого письма выполняется следующая последовательность действий:

- Для нового сообщения создается директория в структуре директорий Dr.Web MailD.
- Конверт и тело сообщения сохраняются в созданной директории.
- (*) Информация о сообщении сохраняется в специальном контейнере.
- Модулю drweb-maild отправляется запрос на проверку письма (возможно, с
  дополнительными параметрами), и от него же получается результат проверки
  (далее результат проверки обозначается как \c action).
- (*) если \c action == ::kACT_Success, то Receiver ожидает обновления информации
  о сообщении в контейнере (максимальное время ожидания задается таймаутом).
  Если время ожидания истекло, то сообщение удаляется из контейнера и action
  присваивается значение ::kACT_Success. В противном случае, если информация
  обновилась до истечения времени ожидания, из контейнера получается информация
  о результате проверки письма, после чего контейнер очищается;
- если \c action == ::kACT_Success, то сообщение блокируется (оптимально делать это
  через действие discard);
- если \c action == ::kACT_Pass, то сообщение отправляется (при необходимости его
  следует предварительно обновить);
- если \c action == ::kACT_Discard или ::kACT_Tempfail или ::kACT_Reject, то сообщение
  соответствующим образом блокируется;
- если \c action == ::kACT_Error или ::kACT_Pass, то удаляется созданная для данного
  письма директория (через DwRsSDK::IProcessingInfo::RemoveDirWithInfo).

Действия, обозначенные через (*), следует выполнять только в том случае, если
требуется поддержка компонентом Receiver сервиса Callback. Если такая поддержка
необходима, следует также запустить дополнительный поток (или потоки), который
будет обрабатывать запросы на проверку письма и соответствующим образом
обновлять контейнер с информацией о сообщении, например, записывая в него
результат проверки письма drweb-maild, а так же его smtp ответ.

Рассмотрим подробнее некоторые из этапов выполнения модуля компонента Receiver.
В ходе рассмотрения будут приводиться примеры на языке C++: применение этих
примеров на языке С потребует тривиальных изменений в коде.

Инициализация лога требуется для вывода как служебной информации, так и
информации об ошибках в работе SDK. В текущей версии SDK поддерживается
вывод лога только через syslog и через консоль.

На этапе инициализации компонента Receiver (DwRsSDK::IReceiver или ::dwrssdk_receiver_t)
также производится и первая попытка установления постоянного соединения
(соединений) с drweb-maild. Если первая попытка заканчивается неудачей, то
следующая попытка предпринимается уже непосредственно при обращении к
drweb-maild с запросом на проверку письма и, соответственно, только к этому моменту
можно будет определить ошибку соединения с drweb-maild.  При попытке
соединения с drweb-maild во флагах следует указывать, поддерживается ли
компонентом Receiver сервис Callback и какие действия он может проводить над
сообщением: может ли он совершить над сообщением действие reject, модифицировать
заголовки, тело сообщения, или конверт перед его отправлением.
Все функции класса IReceiver, реализующего компонент Receiver на C++, являются
потокобезопасными - т.е. их можно вызывать одновременно из нескольких потоков
(кроме DwRsSDK::IReceiver::Initialize).

Так же при инициализализации можно указать уникальный идентификатор
Receiver`а - это позволяет запускать сразу несколько Receiver`ов, каждый из
которых будет создавать сообщения в своей собственной директории и может иметь
свой собственный сервис Callback (которому при инициализации надо будет
указать тот же самый идентификатор). Кроме того, если запущен Sender с тем же
идентификатором, то drweb-maild при необходимости отправить письмо через
Sender будет выбирать именно этот Sender с нужным идентификатором (если
такой Sender не зарегистрирован, то будет использоваться Sender по умолчанию).

Для параметра \a base_dir инициализационной функции сервиса Callback (DwRsSDK::ICallback
или ::dwrssdk_callback_t) следует указывать то же значение, что и для параметра \a base_dir
инициализационной функции компонента  Receiver - значение этих параметров должно совпадать
со значением параметра BaseDir из секции General конфигурационного файла Dr.Web MailD.
Также можно указать уникальный идентификатор, который должен совпадать с тем ID, что был
указан при инициализации Receiver`а.

Также следует зарегистрировать функцию обработчика запросов клиента.

При получении письма на обработку, сначала создается уникальная директория для
его хранения:

 \code
 DwRsSDK::DwPtr<DwRsSDK::IProcessingInfo> pi;
 try{
   if(!(pi=m_sender->SetNewMsgDir()))
     throw domain_error("error in creating new entry for msg on disk");
   ...

 }catch(...){
   if(pi.get()){
      pi->RemoveDirWithInfo();
   }
   //Вхождение для текущего сообщения в контейнер при использовании сервиса Callback.

 }
 \endcode

В этом примере \a m_sender может являться глобальной переменной типа
DwRsSDK::DwPtr<DwRsSDK::IReceiver>, для которой уже была вызвана
функция Initialize. Особое внимание тут следует обратить на корректное
освобождение ресурсов при возникновении ошибок.

Далее следует сохранить новое сообщение в директории, воспользовавшись
классом DwRsSDK::IMail.

   \~english

The execution of the Receiver component module can be divided into several stages:

- Initialization of the DwRsSDK log.
- Initialization of the Receiver component.
- (*)Initialization of the Callback service and registration of the handler function.

Then, for each message the following sequence of actions is performed:

- A directory in the Dr.Web MailD directory structure is created for a new message.
- The envelope and the body of the message are saved to this directory.
- (*) The information on the message is saved in a special container.
- A request to check the message is sent to the drweb-maild module (some additional
  parameters can be added to this request), and check results are received from it
  (hereinafter these results will be denoted as \c action):
- (*) if \c action == ::kACT_Success, then the Receiver component waits for the new
  information on the message in the container (maximum waiting time is set by the
  timeout). If the timeout expires, the message is deleted from the container
  and ::kACT_Success value is assigned to action. Otherwise, if information was
  renewed before timeout expiration, message check results are received from
  the container, and then message is deleted from the container;
- if \c action == ::kACT_Success, the message is blocked (the discard action is
  the best way to do this);
- if \c action == ::kACT_Pass, the message is sent (it should be renewed first,
  if necessary);
- if \c action == ::kACT_Discard, or ::kACT_Tempfail, or ::kACT_Reject, the
  message is blocked;
- if \c action == ::kACT_Error, or ::kACT_Pass, the directory created for the
  message is deleted (via DwRsSDK::IProcessingInfo::RemoveDirWithInfo).

The actions marked with (*) should be performed only if the Callback service
support by the Receiver component is required. If such support is necessary,
additional thread (threads) should be initialized, which will process requests
for the message check and renew the container with information about the
message, i.e. write results of the message check by the drweb-maild module
and its smtp response to it.

Let us consider more closely some stages of execution process of the Receiver
component. Some examples in C++ will follow the current explanation
(to adapt these examples for C language trivial changes in the code are
required).

Initialization of the log is required for successful output of service
information and information about errors in operation of SDK. In current
version of SDK only output to syslog and to the console is supported.

During the initialization of the Receiver component (DwRsSDK::IReceiver
component or dwrssdk_receiver_t) the first attempt to establish a permanent
connection (connections) with drweb-maild module is made.

If the first attempt fails, the next attempt is made when a request is sent for
drweb-maild module to check the message. And therefore only by that time
an error in connection to the drweb-maild module can be detected.

When an attempt to establish connection with drweb-maild module is made,
it must be specified in flags whether the Receiver component supports
the Callback service or not, and what actions can be applied to the message
(e.g. whether the body of the message, its headers or the envelope can be
modified before the message is sent).

All functions of the IReceiver class implementing the Receiver component in C++
are thread-safe (i.e. they may be called from several threads simultaneously),
except for DwRsSDK::IReceiver::Initialize.

Also during initialization a unique identifier for the Receiver component can
be specified. This will allow to run several Receiver components simultaneously,
and each of them will be able to create messages in its personal directory and
have a personal Callback service (with the same unique ID as it was specified for
a corresponding Receiver). If there exists a working Sender component with the
same unique ID, then drweb-maild module will choose this Sender to dispatch a
message (if it does not exist, then the default Sender will be used).

For the \a base_dir parameter of the Callback service's initializing function
(DwRsSDK::ICallback or dwrssdk_callback_t) the same value as for the
\a base_dir parameter of the Receiver component's initializing function
must be specified. Also values of these parameters must match the value
of BaseDir parameter from (General) section of Dr.Web MailD configuration file.

The function of the client's requests handler must also be registered.

When a message is received for processing a unique directory is created, where
this message can be stored:

 \code
 DwRsSDK::DwPtr<DwRsSDK::IProcessingInfo> pi;
 try{
   if(!(pi=m_sender->SetNewMsgDir()))
     throw domain_error("error in creating new entry for msg on disk");

   ...

 }catch(...){
   if(pi.get()){
      pi->RemoveDirWithInfo();
   }
   //Entry for the current message to a container when using the Callback service.
 }
 \endcode

In this example \a m_sender can be a global variable of the
DwRsSDK::DwPtr<DwRsSDK::IReceiver> type, for which Initialize function
has been already called. Special attention must be payed to correct allocation
of resources when errors occur.

Then the new message should be saved to the directory using the
DwRsSDK::IMail class.


    \~
    \code
    string body("From: <test>\r\n\r\nHello world!\r\n");
    DwRsSDK::DwPtr<DwRsSDK::IMail> mail(DwRsSDK::CreateMail(pi->GetDirToSave()));
    if(!mail ||
    !mail->SetSender("programmer@cool.com")||
    !mail->AddRecipient("postmaster")||
    !mail->SaveBody(body.c_str(),body.length())||
    !mail->SaveEnvelope())
      throw domain_error("error in create or save mail msg");

    mail->LogEnvelope();
    \endcode

\~russian

При этом следует учесть, что если тело сообщения велико, то может быть удобнее
работать с ним по частям: для этого можно получить из IMail название файла тела
сообщения (DwRsSDK::IMail::GetBodyFilename), создать файл по указанному адресу
и записывать в него части тела сообщения.

Далее, если реализуется поддержка Callback, то следует сохранить информацию
о сообщении в контейнере (доступном потокам, обрабатывающим соединения
для Callback через DwRsSDK::IServer::Process). Это необходимо сделать до
отправления запроса на проверку письма, для того, чтобы не возникла ситуация,
в которой другой поток уже получил результат проверки письма через Callback,
а в контейнере информации об этом сообщении еще нет.

Далее следует отослать модулю drweb-maild запрос на проверку сообщения и
дождаться результата:

    \~english

Notice, that if the message body is big, it is easier to process it by parts.
For this step-by-step message processing, receive a message body filename
from IMail (DwRsSDK::IMail::GetBodyFilename), create a file with the corresponding
name and write parts of the body to this file.

Then, if Callback service is supported, information about the messages should
be saved to container (also accessible for threads, processing connections for
Callback via DwRsSDK::IServer::Process). This must be done before a request
for message check is sent, to avoid a situation, when another thread has already
received the message check results via Callback, but there is still no information
about this message in the container.

Then a request to check the message should be sent to the drweb-maild module,
and check results should be received from it:
\~


    \code
 DwRsSDK::DwPtr<DwRsSDK::IComponentOptions> options(DwRsSDK::CreateComponentOptions());
 if(was_auth)
    options->Add(DWRSSDK_CO_Auth,0);

 int action(m_sender->SendRequestToMaild(pi.get(),options.get()));
    \endcode
\~russian
В данном примере мы передали вместе с самим письмом еще и набор дополнительных
необязательных параметров. В частности, здесь мы проверяем, прошел ли клиент
при передачи письма авторизацию, и если да, то устанавливаем параметр с
именем DWRSSDK_CO_Auth в опции, что позволит остальным компонентам Dr.Web MailD
(в том числе и получившему письмо Sender`у) использовать эту информацию
при обработке письма. Существует фиксированный набор параметров с
зарегистрированными именами и фиксированной семантикой, которые может
задавать компонент, но он также может задавать свои собственные параметры,
которые должны начинаться с 'X-'.

После этого необходимо обработать результат, возвращенный модулем drweb-maild.

Если \c action == ::kACT_Success, то возможны два варианта дальнейшего
поведения: если не используется Callback, то проще всего присвоить \c action значение
::kACT_Discard и продолжить выполение компонента (полностью переложив
ответственность за письмо на модуль drweb-maild).
В том случае, если сервис Callback поддерживается, необходимо получить от него
результат проверки сообщения. Для этого следует дождатся (с установленным таймаутом
на ожидание) обновления информации о сообщении в контейнере, а затем присвоить
\c action полученнный результат. Если же по истечении таймаута ответ так и не
придет - то установить для \c action значение ::kACT_Discard.

Если \c action == ::kACT_Error, значит произошла ошибка обработки (к примеру,
не удалось соединиться с drweb-maild, или истек таймаут ожидания ответа от него,
или произошла внутренняя ошибка в самом drweb-maild). После получения такого
результата компонент должен удалить директорию, название которой содержится
в \c pi->GetDirToSave(), со всем ее содержимым (например, через
DwRsSDK::IProcessingInfo::RemoveDirWithInfo).

Если \c action == ::kACT_Discard, или ::kACT_Reject, или ::kACT_Tempfail, то данное
сообщение надо соответствующим образом заблокировать. Директорию из
\c pi->GetDirToSave() удалять не надо - это сделает модуль drweb-maild.

В том случае, если \c action ==::kACT_Pass, то следует обновить (при необходимости)
сообщение и отправить его.
Далее приводится пример кода обработки для случая, когда при инициализации IReceiver
не было указано ни одного флага вида \c kMO_Can* :

        \code
    if(action&DwRsSDK::kACT_Pass){
   if(action&DwRsSDK::kACT_WasEnvelopeChange){
      if(!mail->LoadEnvelope())
         throw doamin_error("can not read new envelope");
      //обновление конверта, полученного из \c mail
   }
   if(action&DwRsSDK::kACT_WasBodyModify){
      const char* body;
      int size;
      if(!mail->LoadBody(body,size))
      //обновление тела сообщения через вызов \c mail->GetBodyFilename()
         throw doamin_error("can not read new body");
      //обновление тела сообщения, полученного из \c body
   }
   if(action&DwRsSDK::kACT_WasTopHeaderModify){
      if(!mail->LoadHeaders())
         throw doamin_error("can not read new headers");
      //обновление заголовков, полученных из \c mail
   }
   //Далее можно отправлять сообщение.
 }
    \endcode

В общем случае \c action может принять значение ::kACT_Pass только если при инициализации
компонента Receiver был установлен флаг ::kMO_ReadyForBlockAndSend.

В ответе drweb-maild значения ::kACT_WasEnvelopeChange, ::kACT_WasBodyModify
и ::kACT_WasTopHeaderModify могут прийти только в том случае, если при инициализации
компонента были указаны соответствующие флаги (::kMO_CanSendOnlyWithNewHeaders,
::kMO_CanNotEnvelopeModify, ::kMO_CanNotHeadersModify, ::kMO_CanNotBodyModify).
В том случае, если при инициализации компонента был установлен флаг
::kMO_CanSendOnlyWithNewHeaders (при этом, остальные три флага ::kMO_CanNot* не могли
быть установлены), то из значений ::kACT_Was* может прийти только значение ::kACT_WasTopHeaderModify,
при этом, единственным возможным изменением заголовков является добавление новых полей.
Если пришло значение ::kACT_Pass, то компонент должен вызвать
DwRsSDK::IProcessingInfo::RemoveDirWithInfo для удаления директории.

На этом основной цикл обработки сообщения заканчивается.

Рассмотрим подробнее особенности реализации сервиса Callback.

Сервис Callback может реализовываться как последовательный сервер (вызывается
один поток через IServer::Process), а также как параллельный многопоточный сервер
(вызываются несколько потоков через IServer::Process), по аналогии с реализацией
компонента Sender. На каждый клиентский запрос вызывается зарегистрированная
при инциализации фунция (или, только в C++, также может использоваться зарегистрированный
функтор).

Рассмотрим типичную реализацию этой функции:

    \code
 int callback_func(const char* request,
       int         action,
       const char* smtp_answer,
       const char* from,
       const char* to)
    {
      try{
       if (запрос отсутствует в контейнере)
          return 0;

       if (не удается обработать флаги, заданные в action){
          сообщаем ожидающему потоку, что ответа не будет
          return 0;
       }

       if(action&DwRsSDK::kACT_Pass){
      if(!DwRsSDK::ICallback::PrepareAfterPassReceived(action,from,to))
        throw domain_error("can not prepare pass action");
       }

       сохраняем полученный результат в контейнере для request  (action и smtp_answer)

       сообщаем ожидающему потоку, что ответ получен
      }catch(...){
         сообщаем ожидающему потоку, что ответа не будет
         return 0;
      }
      return 1;
 }
    \endcode

Обратите внимание на вызов DwRsSDK::ICallback::PrepareAfterPassReceived, позволяющий
компоненту получить обратно сообщение (возможно, уже модифицированное) из
БД drweb-maild, для того, чтобы его отправить.

Полностью реализованные примеры компонента Receiver доступны как
на языке C (\ref dwrssdk_receiver_example.c), так и на C++ (\ref
dwrssdk_receiver_example.C). Данные примеры несколько сложнее
тех, что были рассмотрены выше (хотя в них и не используется сервис
Callback), но зато имеют практическое применение (например, на этапе
тестирования или проверки работоспособности всей системы).


    \~english

In the given example we have passed a set of additional optional parameters with the message.
For example, these parameters allow to check whether the client had passed an authorization
when it transferred a message, and if authorization had been passed successfully, then parameter
with DWRSSDK_CO_Auth name is set to options, which in turn allows other Dr.Web MailD components
(including a Sender component, which has received a message) to use this information during
message processing.
There exists a limited set of parameters with registered names and constant semantics, which
may be set by component. But each component is also able to set its own, custom parameters
with names beginning with 'X-'.

After that the result returned by the drweb-maild module should be processed.

If \c action == ::kACT_Success, then two variants of the further behavior are possible:
- if Callback is not used, it may be easier to assign kACT_Discard value to \c action and continue
execution of the component (making drweb-maild module responsible for the message).
- if the Callback service is supported, it is necessary to receive the results of the message
check from it. To receive these results you must wait (during the timeout perion) for the
information on the message to be renewed in the container, and then - assign the received
check results to  \c action. If there is no response by the moment the timeout expires, then
kACT_Discard value must be assigned to \c action.

If \c action == ::kACT_Error, then a processing error has occurred (for example, attempts to
establish connection with drweb-maild module have failed, or timeout for response has expired,
or an internal error has occurred in dwreb-maild). If such result is received, the component
must delete the directory, which name resides in \c pi->GetDirToSave(), and all its contents
(for example, via DwRsSDK::IProcessingInfo::RemoveDirWithInfo).

If \c action == ::kACT_Discard, or ::kACT_Reject, or ::kACT_Tempfail, then the message must be
blocked accordingly. The directory from  \c pi->GetDirToSave() should not be deleted - the
drweb-maild module will do it itself.

If \c action ==::kACT_Pass, the message should be renewed (if necessary) and dispatched.
Below is an example of the processing code for a situation, when none of the flags of the
\c kMO_Can* type were set during the initialization of the IReceiver:

        \code
    if(action&DwRsSDK::kACT_Pass){
   if(action&DwRsSDK::kACT_WasEnvelopeChange){
      if(!mail->LoadEnvelope())
         throw doamin_error("can not read new envelope");
      //renewal of the envelope received from \c mail
   }
   if(action&DwRsSDK::kACT_WasBodyModify){
      const char* body;
      int size;
      if(!mail->LoadBody(body,size))// refreshment of the message body via the call \c mail->GetBodyFilename()
         throw doamin_error("can not read new body");
      //renewal of the message body received from \c body
   }
   if(action&DwRsSDK::kACT_WasTopHeaderModify){
      if(!mail->LoadHeaders())
         throw doamin_error("can not read new headers");
      //renewal of headers received from \c mail
   }
   //Then the message can be sent.
 }
    \endcode

Generally, kACT_Pass can be assigned to \c action only if the kMO_ReadyForBlockAndSend flag
was set during the initialization of the Receiver component.

The kACT_WasEnvelopeChange, kACT_WasBodyModify, and kACT_WasTopHeaderModify values
can come with the drweb-maild module response only if proper flags (kMO_CanSendOnlyWithNewHeaders,
kMO_CanNotEnvelopeModify, kMO_CanNotHeadersModify, kMO_CanNotBodyModify) were specified
during the initialization of the component.
If during the initialization of the component the ::kMO_CanSendOnlyWithNewHeaders flag was set
(and three other ::kMO_CanNot* flags couldn't be set), only the ::kACT_WasTopHeaderModify value
can be received from the ::kACT_Was* values, and the only possible modification of headers in this
case is addition of new fields. If the ::kACT_Pass value was received, then the component must call
DwRsSDK::IProcessingInfo::RemoveDirWithInfo for deletion of the directory.

This is where the main cycle of message processing ends.

Let us consider the implementation of the Callback service in detail.

Like the Sender component, the Callback service can be implemented both as a sequential server
(one thread is called via IServer::Process) and as a parallel multi-thread server (several threads
are called via IServer::Process). A function registered during the initialization is called for each client's
request (or, in C++ only, a registered functor can also be used).

Let us consider a typical implementation of this function:

   \code
 int callback_func(const char* request,
       int         action,
       const char* smtp_answer,
       const char* from,
       const char* to)
    {
      try{
       if (there is no request in a container)
          return 0;

       if (flags set in action cannot be processed){
          report to the waiting thread that there will be no response
          return 0;
       }

       if(action&DwRsSDK::kACT_Pass){
      if(!DwRsSDK::ICallback::PrepareAfterPassReceived(action,from,to))
        throw domain_error("can not prepare pass action");
       }

       Store the received result in a container for request (action and smtp_answer)

       report to the waiting thread that the response is received
      }catch(...){
         report to the waiting thread that there will be no response
         return 0;
      }
      return 1;
 }
    \endcode

Please notice, that DwRsSDK::ICallback::PrepareAfterPassReceived call allows the
component to receive the message back (probably, already modified) from the database
of the drweb-maild module in order to send it.

Fully implemented examples of the Receiver component are available both
in C language (\ref dwrssdk_receiver_example.c), and in C++ (\ref dwrssdk_receiver_example.C).
These examples are a bit more complicated than those described above (though the Callback
service is not used in them), but they have a practical application (for example, during the
testing phase or during the performance check of the whole system).
 \~

*/
/**
   \~russian \page sender Разработка компонента Sender
   \~english \page sender Development of the Sender component
   \~

   \~russian
Процесс выполнения модуля компонента Sender можно разделить на несколько этапов:

- Инициализация лога DwRsSDK.
- Инициализация компонента Sender.
- Регистрация функции обратного вызова для обработки запросов клиентов.

Далее, для каждого письма выполняется зарегистрированная функция, в которой
выполняются основные действия по его обработке.

Рассмотрим подробнее некоторые из этапов выполнения компонента Sender. В ходе
рассмотрения будут приводиться примеры на языке C++: применение этих примеров
на языке С потребует тривиальных изменений в коде.

Инициализация лога требуется для вывода как служебной информации,
так и информации об ошибках в работе SDK. В текущей версии SDK
поддерживается вывод лога только через syslog и через консоль.

При инициализации компонента Sender (DwRsSDK::ISender или ::dwrssdk_sender_t)
параметр \a base_dir должен совпадать со значением параметра BaseDir из секции
General конфигурационного файла Dr.Web MailD.
Также можно указать уникальный идентификатор, использование которого позволит
запускать сразу несколько компонентов Sender.
drweb-maild будет выбирать, какому из Sender'ов отправить письмо, на основе уникального
идентификатора Receiver`а, который отправил сообщение (таким образом, не имеет смысла
задавать в Sender`е уникальный идентификатор, если не запущен Receiver с аналогичным
уникальным идентификатором, так как Sender в этом случае просто не будет получать писем).
Компонент Sender может реализовываться как последовательный сервер (вызывается один
поток через IServer::Process), и как параллельный многопоточный сервер (вызывается несколько
потоков через IServer::Process). Для каждого запроса клиента вызывается зарегистрированная
при инциализация функция (или, только в С++, может использоваться зарегистрированный функтор).

Рассмотрим пример реализации функтора обработки запросов клиента:

  \~english

The execution of the Sender component module can be divided into several stages:

- Initialization of the DwRsSDK log.
- Initialization of the Sender component.
- Registration of the callback function for processing of clients' requests.

Then a registered function, in which main actions for message processing are
performed, is executed for each message.

Let us consider more closely some stages of execution process of the Sender
component. Some examples in C++ will follow the current explanation
(to adapt these examples for C language trivial changes in the code are
required).

Initialization of the log is required for successful output of service
information and information about errors in operation of SDK. In current
version of SDK only output to syslog and to the console is supported.

During the initialization of the Sender component (DwRsSDK::ISender component
or ::dwrssdk_sender_t) the \a base_dir parameter must match the value
of the BaseDir parameter from General section of the Dr.Web MailD
configuration file. Also a unique ID can be specified, which will allow to run
several Sender components simultaneously.
The Sender component can be implemented both as a sequential server
(one thread is activated via IServer::Process) and a parallel multi-thread server
(several threads are activated via IServer::Process). For each client's request
a function registered during the initialization is activated (in C++ a registered functor can be used).

Below is an example of the implementation of client's request processing functor:

 \~

   \code
    struct SenderFunctor:public DwRsSDK::DwSenderFunctor
 {
  int operator()(const char* dir,
        const char* orig_data,
        DwRsSDK::IComponentOptions* comp_opt,
        bool need_remove,
        int  options,
        int  timeout)
  {
   try{
    DwRsSDK::DwPtr<DwRsSDK::IMail> mail(DwRsSDK::CreateMail(dir));

    if(!mail->LoadEnvelope()){
     throw domain_error("Can not read enevlope info for msg "+string(orig_data));
    }
    mail->LogEnvelope();

    if(need_remove){
       DwRsSDK::RemoveDirWithInfo(dir);
    }

   } catch (exception&){
    return DwRsSDK::kACT_Reject;
   }
   return DwRsSDK::kACT_Pass;
  }

 };

   \endcode

\~russian

В данном примере сначала происходит считывание конверта письма и
вывод его в лог. Далее, при необходимости, директория с письмом удаляется.

Такой компонент представляет собой "черную дыру", которая никаких писем
не отправляет, а, по возможности, их стирает. Следует отметить обязательность
удаления директории, заданной в \a dir - первом параметре функции, если
установлен флаг need_remove и было возвращено значение DwRsSDK::kACT_Pass.
В противном случае на диске будут накапливаться никому не нужные временные
файлы. Так же видно, что третьим параметром comp_opt переданы все параметры,
которые задали предыдущие компоненты системы для данного сообщения.

Полностью реализованные примеры компонента Sender доступны как на языке C
(\ref dwrssdk_sender_example.c), так и на C++ (\ref dwrssdk_sender_example.C).
Данные примеры несколько сложнее тех, что были рассмотрены выше, но зато
имеют практическое применение (например, на этапе тестирования или проверки
работоспособности всей системы).

    \~english

In the given example the envelope of the message is processed and output to the
log first. Then, if necessary, the directory with the message is deleted.
Such component is a "black hole" that never sends any messages, but deletes
them, if possible. Please note, that if the need_remove flag is set and the
DwRsSDK::kACT_Pass value was returned, then the directory specified in \a dir
- the first parameter of the function - must be obligatory deleted. Otherwise
unnecessary temporary files will be accumulated on disk. You may also notice,
that the third parameter comp_opt transfers all parameters specified by previous
components for a certain message.

Fully implemented examples of the Sender component are available both in C
(\ref dwrssdk_sender_example.c), and in C++ (\ref dwrssdk_sender_example.C).
These examples are a bit more complicated than those described above, but
they have a practical application (for example, during the test stage or for check
of the operability of the whole system).
\~

   */

/**\example dwrssdk_receiver_example.c

   \~russian

Пример модуля компонента Receiver на языке C.

Модуль отсылает все файлы с расширением .eml из директории, заданной в командной
строке, в drweb-maild. Если директория не задана, то производится поиск сообщений
в текущей директории. В командной строке также может быть указан путь к конкретному
файлу - в этом случае будет отослан только указанный файл. Информация о конверте письма
будет определяться из самого письма. По ее получении будет предпринята попытка
нахождения и разбора заголовков From и To.

   \~english

Example of the module of the Receiver component in C.

This module sends all files with the .eml extension from the directory specified in the
command line to the drweb-maild module. If the directory is not set, search for messages
will be performed in the current directory. A path to a certain file can be specified in the
command line. In this case only this file will be sent. Information about the envelope
will be determined from the message itself, and an attempt to find and parse From and To
headers will be made.

 \~

 */

/**\example dwrssdk_receiver_example.C

   \~russian

Пример модуля компонента Receiver на языке C++.

Модуль отсылает все файлы с расширением .eml из директории, заданной в командной
строке, в drweb-maild. Если директория не задана, то производится поиск сообщений
в текущей директории. В командной строке также может быть указан путь к конкретному
файлу - в этом случае будет отослан только указанный файл. Информация о конверте письма
будет определяться из самого письма. По ее получении будет предпринята попытка
нахождения и разбора заголовков From и To.

   \~english

Example of the module of the Receiver component in C++.

This module sends all files with the .eml extension from the directory specified in the
command line to the drweb-maild module. If the directory is not set, search for messages
will be performed in the current directory. A path to a certain file can be specified in the
command line. In this case only this file will be sent. Information about the envelope
will be determined from the message itself, and an attempt to find and parse From and To
headers will be made.

\~

*/

/**\example dwrssdk_sender_example.c

   \~russian

Пример модуля компонента Sender на языке C.

Работает в двух различных режимах, в зависимости от того задана ли в командной
строке директория, или нет. Если директория задана, то каждое сообщение, на которое
пришел запрос на отправку, будет сохранено в этой директории под уникальным
именем (вместе с конвертом сообщения, который будет сохранен под тем же именем,
только с расширением .env). В противном случае запросы будут только фиксироваться
в логах. Если задан параметр -p, то тело каждого сообщения будет выводиться на консоль.

   \~english

Example of the module of the Sender component in C.

It operates in two different modes, depending on whether the directory is set in the command
line, or not. If the directory is set, then each message, for which a request for sending
was received, will be stored in this directory under a unique name (together with the
envelope of the message, which will be stored under the same name and with the .env
extension). Otherwise, requests will be output to logs only. If -p parameter is specified, then
body of each message will be output to console.

\~

 */

/**\example dwrssdk_sender_example.C

   \~russian

Пример модуля компонента Sender на языке C++.

Работает в двух различных режимах, в зависимости от того, задана ли в командной
строке директория, или нет. Если директория задан, то каждое сообщение, на которое
пришел запрос на отправку, будет сохранено в этой директории под уникальным
именем (вместе с конвертом сообщения, который будет сохранен под тем же именем,
только с расширением .env). В противном случае запросы будут только фиксироваться
в логах. Если задан параметр -p, то тело каждого сообщения будет выводиться на консоль.

   \~english

Example of the module of the Sender component in C++.

It operates in two different modes, depending on whether the directory is set in the command
line, or not. If the directory is set, then each message, for which a request for sending
was received, will be stored in this directory under a unique name (together with the
envelope of the message, which will be stored under the same name and with the .env
extension). Otherwise, requests will be output to logs only. If -p parameter is specified, then
body of each message will be output to console.

\~
 */

#endif
