/*--------------------------------------------------------------------------
 *  Copyright (c) 2006-2012 Igor Daniloff.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found in this directory.
 *------------------------------------------------------------------------*/

#ifndef DW_RS_SDK_ICALLBACK_H_
#define DW_RS_SDK_ICALLBACK_H_

#include "DwRsSDK_IServer.H"

/**\file
   \~russian Поддержка сервиса Callback. Данный сервис позволяет
   обрабатывать сообщения компонентом Receiver даже в тех случаях, когда
   drweb-maild не может сразу вернуть результат проверки сообщения (например,
   если некоторые плагины находятся в очереди AfterQueueFilters).
   \~english Support of the Callback service. This service allows to process
   messages by the Receiver component even when drweb-maild cannot immediately
   return the result of the message check (for example, if some plug-ins are in the
   AfterQueueFilters queue). \~
*/

/** \addtogroup C_INTERFACE
 *  @{
 */

/**\copydoc DwRsSDK::ICallback
 */
typedef void* dwrssdk_callback_t;

/**
   \~russian Регистрация функции обратного вызова для обработки запросов от клиентов.
   Должна вызываться перед любым вызовом функции обработки запросов клиента.
   \~english Registration of the callback function for processing queries from clients.
   Must be called before any call of the client's query processing function. \~
   \sa DwRsSDK::IServer::Process DwRsSDK_ProcessCallback DwRsSDK_ProcessSender

   \param[in] c - \~ \copydoc dwrssdk_callback_t

   \param[in] cp - \~russian Указатель на функцию для обработки.
   \~english Pointer to the function for processing. \~

 */
DWRSSDK_EXTERN int  DwRsSDK_RegisterProcessingFuncCallback(dwrssdk_callback_t c,dwrssdk_callback_processing_t cp);

/**\copydoc DwRsSDK::CreateCallback
 */
DWRSSDK_EXTERN dwrssdk_callback_t DwRsSDK_CreateCallback(void);
/**\copydoc DwRsSDK::DeleteCallback
 */
DWRSSDK_EXTERN void DwRsSDK_DeleteCallback(dwrssdk_callback_t r);
/**\~russian Инициализация настройки сервера. Должна вызываться
    перед любой функцией данного класса и функциями его производных
    классов. Допускается повторный вызов (к примеру, при перечитывании
    конфигурации), но при это все потоки, вызывающие IServer::Process,
    должны быть заблокированы через IServer::Block. После успешной
    инициализации потоки следует снова разблокировать через IServer::Unblock.
    \~english Initialization of the server settings. Must be called before any function
    of the given class and functions of its derived classes. Repeated call is allowed
    (for example, to re-read the configuration), but all the threads calling for IServer::Process
    must be blocked via IServer::Block. After successful initialization these threads
    should be unblocked via IServer::Unblock. \~

    \param[in] c - \~ \copydoc dwrssdk_callback_t

    \param[in] id - \~russian уникальный идентификатор компонента. Будет
    создан сокет в /var/drweb/ipc (по умолчанию), в состав имени которого
    будет входить данный идентификатор.
   \~english unique identifier of the component. It will be used as a part of socket name
    created in /var/drweb/ipc by default.
    \~

    \param[in] base_dir - \~russian строка с адресом базовой директории MailD (параметр
    BaseDir из секции General конфигурационного файла Dr.Web MailD). Должна
    заканчиваться знаком '/'. При установке по умолчанию значение должно быть
    равно '/var/drweb/'.
    \~english a string with the path to the base directory of MailD (the BaseDir parameter from General section
    of Dr.Web MailD configuration file). Must end with the '/' symbol. If Dr.Web MailD was
    installed with default settings, the value of this parameter is equal to '/var/drweb/'. \~

    \param[in] timeout - \~russian значение таймаута (в секундах) на
    работу с одним клиентом.
    \~english value of the timeout (in seconds) for operation with one client. \~

    \param[in] stat - \~russian собирать ли статистику.
     \~english  whether to collect statistics. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
 */
DWRSSDK_EXTERN int DwRsSDK_InitializeCallback(dwrssdk_callback_t c,
                                              const char* base_dir,
                                              const char* id,
                                              int  timeout,
                                              int  stat
                                              );
/**\~russian Сохраняет статистику по использованию соединений в
       заданный файл.
    \~english Collects statistics on the use of connections to the specified file.\~

   \param[in] c - \~ \copydoc dwrssdk_callback_t

   \param[in] file - \~russian имя файла, в который должна быть сохранена статистика.
    \~english name of the file to which statistics is saved.\~

   \return \~russian 1 - при успехе, 0 - при неудаче.
    \~english 1 - at success, 0 - at failure.  \~

*/
DWRSSDK_EXTERN int DwRsSDK_DumpStatCallback(dwrssdk_callback_t c,const char* file);

/**
   \~russian Функция блокирует все потоки работающие в IServer::Process
     до вызова IServer::Unblock.
       \~english The function blocks all threads working in IServer::Process
     until IServer::Unblock is called. \~

       \param[in] c - \~ \copydoc dwrssdk_callback_t

       \param[in] timeout - \~russian таймаут (в секундах) на выполнение
       блокировки.
       \~english timeout (in seconds) for blocking. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~

*/
DWRSSDK_EXTERN int DwRsSDK_BlockCallback(dwrssdk_callback_t c,int timeout);
/**
   \copydoc DwRsSDK::IServer::Unblock

   \param[in] c - \~ \copydoc dwrssdk_callback_t
*/
DWRSSDK_EXTERN void DwRsSDK_UnblockCallback(dwrssdk_callback_t c);
/**\~russian Посылает запрос на остановку потока, работающего в IServer::Process.
      Данный поток выходит из IServer::Process с кодом ::PR_StopThread. Эта функция
      не ждет завершения работы потока - такую проверку должно выполнить само приложение.
       \~english Sends a query to stop a thread working in IServer::Process.
       This thread exits the IServer::Process with the PR_StopThread code. This function does not wait for the
       thread to terminate its operation - the application must check this itself. \~

   \param[in] c - \~ \copydoc dwrssdk_callback_t

   \param[in] tid - \~russian идентификатор потока для остановки.
       \~english identifier of a thread for termination. \~
*/
DWRSSDK_EXTERN void DwRsSDK_StopThreadCallback(dwrssdk_callback_t c,
                                               const void* tid);

/**
   \~russian Обработка запросов клиента. Должна вызываться в бесконечном цикле.
       При завершении с кодом ::PR_StopThread приложение должно выйти из цикла и
       завершить выполнение потока.
       \~english Processing of client queries. Must be called in an infinite loop. If ended with
       the ::PR_StopThread code, the application must exit the loop and terminate execution
       of the thread. \~

   \param[in] c - \~ \copydoc dwrssdk_callback_t

   \param[in] timeout - \~russian таймаут, в течение которого нужно ожидать запросов
       от клиентов. 0 - не устанавливать таймаут. Если произошел таймаут, то будет возвращено
       ::PR_StopThread.
       \~english timeout for awaiting requests from clients. 0 - timeout is not set. If timeout is expired, then
       ::PR_StopThread is returned. \~

   \return \~russian Код завершения обработки.
       \~english Code of processing termination. \~
*/
DWRSSDK_EXTERN DwRsSDK_ProcessRes DwRsSDK_ProcessCallback(dwrssdk_callback_t c,
                                                          int timeout);

/** @} */

/**\copydoc DwRsSDK::ICallback::PrepareAfterPassReceived
 */
DWRSSDK_EXTERN int DwRsSDK_PrepareAfterPassReceived(int action,
                                                    const char* from,
                                                    const char* to);


DWRSSDK_BEGIN

#ifdef __cplusplus

class DwCallbackFunctor;

/**\~russian Реализация сервиса Callback.
\~english Implementation of the Callback service. \~
 */
class ICallback:virtual public IServer
{
public:
 virtual ~ICallback(){}

 /**\~russian Регистрация функции обратного вызова для обработки
   запросов от клиентов. Должна вызываться перед любым вызовом
   функции обработки запросов клиента.
   \~english Registration of the callback function for processing queries from
   clients. Must be called before any call of the client's query processing function. \~
    \sa DwRsSDK::IServer::Process DwRsSDK_ProcessCallback DwRsSDK_ProcessSender

    \param[in] cp - \~russian Указатель на функцию для обработки.
   \~english Pointer to the function for processing. \~
  */
 virtual bool RegisterProcessingFunc(dwrssdk_callback_processing_t cp)=0;
 /**\~russian Регистрация функтора для обработки запросов от
    клиентов. Должен вызываться перед любым вызовом функции
    обработки запросов клиента.
    \~english Registration of the functor for processing queries from clients.
    Must be called before any call of the client's query processing function. \~

    \sa DwRsSDK::IServer::Process

    \param[in] cp - \~russian Указатель на функтор для обработки.
    \~english Pointer to the functor for processing. \~
  */
 virtual bool RegisterProcessingFunc(DwCallbackFunctor* cp)=0;

 /**\~russian Функция перемещения/копирования директории с сообщением после получения ::kACT_Pass.
    Должна вызываться из функции обратного вызова ::dwrssdk_callback_processing_t.
    \~english Function for moving/copying a directory with the message after receipt of ::kACT_Pass.
    Must be called from the callback function ::dwrssdk_callback_processing_t. \~

    \param[in] action - \~russian параметр \a action из ::dwrssdk_callback_processing_t.
    \~english parameter \a action from ::dwrssdk_callback_processing_t. \~
    \param[in] from - \~russian параметр \a from из ::dwrssdk_callback_processing_t.
    \~english parameter \a from from ::dwrssdk_callback_processing_t .\~
    \param[in] to - \~russian параметр \a to из ::dwrssdk_callback_processing_t.
    \~english parameter \a to from ::dwrssdk_callback_processing_t. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
  */
 static  bool PrepareAfterPassReceived(int action,const char* from,const char* to);
};

/**\~russian Функтор обработки запросов от клиентов для сервиса Callback.
\~english Functor for processing queries from clients for the Callback service. \~
 */
class DwCallbackFunctor {
public:
 /**\~russian Регистрация функции обработки.
 \~english Registration of the processing function. \~

    \param[in] t - \~russian Указатель на функцию обработки.
    \~english Pointer to the processing function. \~
  */
 DwCallbackFunctor(dwrssdk_callback_processing_t t=0):
  m_caller(t)
 {}

 /**\~russian Прокси-вызов обработчика.
 \~english Proxy call of the handler. \~
    \sa dwrssdk_callback_processing_t
  */
 virtual bool operator()(const char* request,
                         int         action,
                         const char* smtp_answer,
                         const char* from,
                         const char* to)
 {
  if(m_caller)
   return m_caller(request,action,smtp_answer,from,to);
  else
   return false;
 }

 virtual ~DwCallbackFunctor() {}
private:
 dwrssdk_callback_processing_t m_caller;
};

/**\~russian Функция создает новый объект поддержки сервиса Callback.
\~english The function creates new object of the Callback service support. \~
   \return \~russian Созданный объект, или NULL при ошибке.
   \~english Created object or NULL, if an error occurs.  \~
*/
DWRSSDK_EXTERN ICallback*   CreateCallback();
/**\~russian Функция освобождает память объекта поддержки сервиса Callback.
\~english The function deallocates memory of the Callback service support object. \~

   \param[in]   r - \~russian Указатель на объект для освобождения.
   \~english Pointer to the object for deallocation. \~
*/
DWRSSDK_EXTERN void         DeleteCallback(ICallback* r);

#endif

DWRSSDK_END

#endif
