/*---------------------------------------------------------------------------
 * Copyright 2008-2012 Igor Daniloff.
 *
 * Following source code is the property of Doctor Web.
 * Use is subject to license terms.
 * The reproduction in any form without written permission of
 * Doctor Web and proper attribution is strongly prohibited.
 ----------------------------------------------------------------------------*/

#ifndef DW_RS_SDK_ICOMPONENT_OPTIONS_H_
#define DW_RS_SDK_ICOMPONENT_OPTIONS_H_

/** \file
 \~russian Поддержка передачи дополнительных настроек между компонентами Dr.Web MailD.
 \~english Support of transfer of additional settings between Dr.Web MailD components. \~
*/

#include "DwRsSDK_defs.H"
#include "DwRsSDK_IUnknown.H"

/** \addtogroup C_INTERFACE
 *  @{
 */

/** @{ \~russian Зарезервированные названия параметров.
           \~english Reserved parameter names. \~*/
#define DWRSSDK_CO_ClientIP "C"/**< \~russian Строка с IP адресом клиента, который
     передал письмо в систему. Данное значение в компоненте drweb-maild имеет приоритет
     перед значением IP адреса, указанным в последнем заголовке Receiver.
    \~english String with IP address of the client, that has passed the message to the system.
    In drweb-maild component this value has priority over the value of IP address specified in
    the last header of the Receiver.  \~ */
#define DWRSSDK_CO_ClientPort "Cp"  /**< \~russian Номер порта клиента, который передал
     письмо в систему.
    \~english Number of the port of the client, that has passed the message to the system. \~ */
#define DWRSSDK_CO_Auth "A"  /**< \~russian Клиент, передавший письмо, прошел успешную
     авторизацию. В качестве значения может быть указан механизм, с помощью которого была
     выполнена авторизация.
   \~english Authorization of the client, that has passed the message, was successful. An authorization
    procedure may be specified as a value. \~ */
#define DWRSSDK_CO_ServerUnixSock "Sus"  /**< \~russian Имя UNIX-сокета, на который пришло письмо.
   \~english Name of the UNIX-socket, that has received the message. \~ */
#define DWRSSDK_CO_ServerIP "S" /**< \~russian Строка с IP адресом интерфейса, на котором было
    принято соединение.
    \~english String with IP address of the interface, that has accepted a connection. \~ */
#define DWRSSDK_CO_ServerPort "Sp"  /**< \~russian Номер порта, на который пришло письмо.
    \~english Number of the port, that has received the message. \~ */

#define DWRSSDK_CO_RsID "ID" /**< \~russian Идентификатор Receiver`а, сгенерировавшего письмо.
                              Данный параметр автоматически устанавливается функцией DwRsSDK::IReceiver::SendRequestToMaild,
                              если в DwRsSDK::IReceiver::Initialize был указан соответствующий идентификатор.
\~english Identifier of a Receiver which has generated a message. This parameter is set automatically by
   DwRsSDK::IReceiver::SendRequestToMaild function, if a corresponding identifier has been specified in
   DwRsSDK::IReceiver::Initialize.  \~  */

#define DWRSSDK_CO_SenderAddress "SA" /**< \~russian Предпочтительный адрес отправления данного письма.
 Если Sender поддерживает данную настройку, то он должен пытаться отправить письмо именно на этот адрес
 независимо от своих настроек маршрутизации. Формат адреса аналогичен формату адреса в
 конфигурационных файлах Dr.Web MailD: например local:/path/to/unix/socket или inet:3003\@localhost.
\~english Preferred dispatch addres for a given message. If the Sender supports this setting, it should try to dispatch
 the message exactly to the specified address regardless of its routing settings. Address format is similar to the format
 of addresses in Dr.Web Maild configuration files: e.g. local:/path/to/unix/socket or inet:3003\@localhost..\~  */

#define DWRSSDK_CO_MsgId "I" /**< \~russian Уникальный идентификатор, связанный с сообщением,
 который будет выводится в логах при обработке сообщения.
 \~english A unique identifier, connected with the message. It will be specified in logs during message processing. \~  */

#define DWRSSDK_CO_Notify "N" /**< \~russian Дополнительная информация по высылаемым отчетам.
 \~english Additional information about dispatched reports. \~  */

#define DWRSSDK_CO_MDClientsMap "M" /**< \~russian список идентификаторов Клиентов, разделенный запятыми для каждого получателя в конверте \~english FIXME \~  */

#define DWRSSDK_CO_PluginsRegexForCheck "P" /**< \~russian perl regex для имен плагинов, которыми надо проверить письмо. Если параметр указан и пустой, то письмо пройдет без проверки. Если параметр не указан, то он принимается равным '.' (без кавычек) - т.е. сообщение будет проверено всеми плагинами \~english FIXME \~  */

/** @} */


/** \copydoc DwRsSDK::IComponentOptions */
typedef void* dwrssdk_component_options_t;


/**\~russian Добавление нового параметра.
         \~english Addition of the new parameter. \~

 \param[in] c - \~ \copydoc dwrssdk_component_options_t

 @param[in] name - \~russian имя параметра. Можно задавать
     свои собственные параметры, начиная их с X-.
     \~english parameter name. Custom parameters can be specified,
      with the name beginning with X-.  \~

 @param[in] val - \~russian значение параметра. Может быть
     равно 0 - тогда будет передаваться только имя параметра.
     Значение не должно содержать знака \n.
     \~english parameter value. May be equal to 0 - then only parameter
     name will be passed. Value must not contain \n symbol.  \~

 @return \~russian 1 - при успехе, 0 - при неудаче.
     \~english 1 - at success, 0 - at failure. \~
 */
DWRSSDK_EXTERN int DwRsSDK_ComponentOptions_Add(dwrssdk_component_options_t c,const char* name,const char* val);
/**\~russian Поиск значения параметра с заданным именем.
 \~english Search for a value of parameter with a specified name. \~

 \param[in] c - \~ \copydoc dwrssdk_component_options_t

 @param[in] name - \~russian имя параметра. Значение не должно быть равно 0.
\~english parameter name. Value must not be equal to zero. \~

 @return \~russian Указатель на указатель на параметр. Полученную строку нельзя
 редактировать и удалять через free. Если параметр равен нулю, то будет производиться
 только проверка на наличие параметра.
\~english Pointer to the pointer to parameter. Received string cannot be edited and deleted via free.
 If parameter value is equal to zero, then only check for the presence of parameter will be performed.\~

 */
DWRSSDK_EXTERN const char* DwRsSDK_ComponentOptions_Find(dwrssdk_component_options_t c, const char* name);
/** \~russian Замена старого значения на новое. Если старого значения не
     было, то будет добавлено новое.
     \~english Old value is changed with the new one. If there was no old value,
     then the new one will be simply added. \~

 \param[in] c - \~ \copydoc dwrssdk_component_options_t

 @param[in] name - \~russian имя параметра. Можно задавать
     свои собственные параметры, начиная их с X-.
     \~english parameter name. Custom parameters can be specified,
      with the name beginning with X-.  \~

 @param[in] val - \~russian значение параметра. Может быть равно 0 - тогда
     параметр будет удален. Значение не должно содержать знака \n.
     \~english parameter value. May be equal to 0 - then parameter will be deleted.
     Value must not contain \n symbol. \~

 @return \~russian True - при успехе, false - при неудаче.
     \~english True - at success, false - at failure. \~
*/
DWRSSDK_EXTERN int DwRsSDK_ComponentOptions_Replace(dwrssdk_component_options_t c, const char* name,const char* val);
/**\~russian Доступ к представлению. Значение нельзя изменять или удалять.
     Значение имеет бинарную форму (могут быть нули) - его длину можно узнать
     через DwRsSDK_ComponentOptions_GetSize.
        \~english Access to representation. Value cannot be changed or deleted.
      Value is presented in binary form (there may be 0) - information about its length
      can be obtained via DwRsSDK_ComponentOptions_GetSize. \~

 \param[in] c - \~ \copydoc dwrssdk_component_options_t

 @return \~russian Указатель на данные.
           \~english Pointer to data. \~
 */
DWRSSDK_EXTERN const char* DwRsSDK_ComponentOptions_GetData(dwrssdk_component_options_t c);
/**\~russian Возвращает размер представления.
         \~english Returns representation size. \~

 \param[in] c - \~ \copydoc dwrssdk_component_options_t

 @return \~russian Неотрицательный размер представления (может быть равно 0).
         \~english Non-negative size of representation (may be equal to 0). \~
 */
DWRSSDK_EXTERN size_t     DwRsSDK_ComponentOptions_GetSize(dwrssdk_component_options_t c);

/** \~russian Создание нового объекта dwrssdk_component_options_t.
     Значение должно быть освобождено через DwRsSDK_DeleteComponentOptions.
       \~english Creation of the new object dwrssdk_component_options_t.
      Value must be deallocated via DwRsSDK_DeleteComponentOptions.\~

 \param[in] data - \~russian представление значения компонента (может быть равно 0).
  \~english component's value representation (may be equal to 0). \~

  \param[in] length - \~russian длина представления значения компонента.
           \~english length of component's value representation.\~

 @return \~russian Указатель на вновь созданный объект.
            \~english Pointer to the newly created object. \~
*/
DWRSSDK_EXTERN dwrssdk_component_options_t DwRsSDK_CreateComponentOptions(const char* data,size_t length);

/** \~russian Освобождает память выделенную через \sa DwRsSDK_CreateComponentOptions.
    \~english Deallocates memory allocated via \sa DwRsSDK_CreateComponentOptions. \~

 @param options - \~russian Указатель на удаляемый объект.
    \~english Pointer to the object for deletion. \~
*/
DWRSSDK_EXTERN void DwRsSDK_DeleteComponentOptions(dwrssdk_component_options_t options);

/** @} */

DWRSSDK_BEGIN

#ifdef __cplusplus

/**\~russian Класс поддержки параметров между компонентами MailD.
    Данные параметры будут передаваться вместе с сообщением насквозь
    через все компоненты MailD.
   \~english Class, providing support for parameters through various MailD components.
    These parameters will be passed with the message through all MailD components. \~
*/
class IComponentOptions:virtual public IUnknown
{
public:
    virtual ~IComponentOptions() {}

    /** \~russian Добавление нового параметра.
         \~english Addition of the new parameter. \~

     @param[in] name - \~russian имя параметра. Можно задавать
     свои собственные параметры, начиная их с X-.
     \~english parameter name. Custom parameters can be specified,
      with the name beginning with X-.  \~

     @param[in] val - \~russian значение параметра. Может быть
     равно 0 - тогда будет передаваться только имя параметра.
     Значение не должно содержать знака \\n.
     \~english parameter value. May be equal to 0 - then only parameter
     name will be passed. Value must not contain \\n symbol.  \~

     @return \~russian True - при успехе, false - при неудаче.
     \~english True - at success, false - at failure. \~
    */
    virtual bool Add(const char* name,const char* val)=0;

    /** \~russian Поиск значения параметра с заданным именем.
 \~english Search for a value of parameter with a specified name. \~

     @param[in] name - \~russian имя параметра. Значение не должно
     быть равно 0.
     \~english parameter name. Value must not be equal to 0. \~

     @return \~russian Строку значения параметра, или 0, если параметр не был
     найден. Полученную строку нельзя редактировать и удалять через delete/free.
     \~english String with parameter value, or 0, if parameter was not found. Received
     string cannot be edited or deleted via delete/free.  \~

    */
    virtual const char* Find( const char* name)const=0;

    /** \~russian Замена старого значения на новое. Если старого значения не
     было, то будет добавлено новое.
     \~english Old value is changed with the new one. If there was no old value,
     then the new one will be simply added. \~

     @param[in] name - \~russian имя параметра. Можно задавать свои
     собственные параметры, начиная их с X-.
      \~english parameter name. Custom parameters can be specified, with the name
      beginning with X-. \~

     @param[in] val - \~russian значение параметра. Может быть равно 0 - тогда
     параметр будет удален. Значение не должно содержать знака \\n.
     \~english parameter value. May be equal to 0 - then parameter will be deleted.
     Value must not contain \\n symbol. \~

     @return \~russian True - при успехе, false - при неудаче.
     \~english True - at success, false - at failure. \~
    */
    virtual bool Replace(const char* name,const char* val)=0;

    /** \~russian Доступ к представлению. Значение нельзя изменять или удалять.
     Значение имеет бинарную форму (могут быть нули) - его длину можно узнать
     через DwRsSDK::IComponentOptions::GetSize.
        \~english Access to representation. Value cannot be changed or deleted.
      Value is presented in binary form (there may be 0) - information about its length
      can be obtained via DwRsSDK::IComponentOptions::GetSize. \~


     @return \~russian Указатель на данные.
           \~english Pointer to data. \~
    */
    virtual const char* GetData()const=0;

    /**  \~russian Возвращает размер представления.
         \~english Returns representation size. \~


     @return \~russian Неотрицательный размер представления (может быть равно 0).
         \~english Non-negative size of representation (may be equal to 0). \~
    */
    virtual size_t     GetSize()const=0;
};


/** \~russian Создание нового объекта IComponentOptions. Значение должно быть
     освобождено через DwRsSDK::DeleteComponentOptions.
     \~english Creation of the new object IComponentOptions. Value must be
     deallocated via DwRsSDK::DeleteComponentOptions. \~

 \param[in] data - \~russian представление значения компонента (может быть равно 0).
  \~english component's value representation (may be equal to 0). \~

 \param[in] length - \~russian длина представления значения компонента.
  \~english length of component's value representation.\~

 @return \~russian Указатель на вновь созданный объект.
  \~english Pointer to the newly created object. \~
*/
IComponentOptions* CreateComponentOptions(const char* data=0,size_t length=0);

/** \~russian Освобождает память, выделенную через DwRsSDK::CreateComponentOptions.
     \~english Deallocates memory allocated via DwRsSDK::CreateComponentOptions. \~

 @param options - \~russian Указатель на удаляемый объект.
   \~english Pointer to the object for deletion. \~
*/
void DeleteComponentOptions(IComponentOptions* options);

#endif

DWRSSDK_END

#endif
