/*--------------------------------------------------------------------------
 *  Copyright (c) 2006-2012 Igor Daniloff.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found in this directory.
 *------------------------------------------------------------------------*/

#ifndef DW_RS_SDK_IPROCESSING_INFO_H_
#define DW_RS_SDK_IPROCESSING_INFO_H_

/**\file
   \~russian Поддержка внутренней информации при взаимодействии Receiver и drweb-maild.
   \~english Support of internal information during interaction between the Receiver and drweb-maild. \~
 */

#include <sys/types.h>
#include <pwd.h>

#include "DwRsSDK_defs.H"
#include "DwRsSDK_IUnknown.H"

/** \~russian @defgroup C_INTERFACE Интерфейс для языка C.
 *  \~english @defgroup C_INTERFACE Interface for C language.
 *  @{
 */

/**\~russian Тип, в котором хранится указатель на внутреннюю информацию.
\~english The type in which the pointer to the internal information is stored. \~*/
typedef void* dwrssdk_processing_info_t;

/**
   \~russian Если Receiver запущен с правами администратора (root),
    а drweb-maild - с правами другого пользователя, то необходимо установить
    требуемые права на файлы с сообщением перед передачей их на
    проверку drweb-maild.
    \~english If the Receiver is started with root privileges, and drweb-maild - with
    privileges of some other user, then it is necessary to set the required permissions
    to files with the message before these files are passed to drweb-maild for check. \~

   \param[in] pi - \~russian указатель на внутреннюю информацию.
   \~english pointer to internal information. \~

   \param[in] uid - \~russian идентификатор пользователя, который надо установить на файлы.
    Укажите значение -1, если идентификатор менять не надо.
    \~english user identifier, which must be specified as files owner.
    Specify -1, if there is no need to change the identifier. \~

   \param[in] gid - \~russian идентификатор группы, который надо установить на файлы.
    Укажите значение -1, если идентификатор менять не надо.
    \~english group identifier which must be specified as files owner.
    Specify -1, if there is no need to change the identifier. \~

   \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
   \~english 1 - the action was successfully completed, 0 - an error occurred. \~
*/
DWRSSDK_EXTERN int DwRsSDK_SetChown(dwrssdk_processing_info_t pi,uid_t uid,gid_t gid);
/**\copydoc DwRsSDK::IProcessingInfo::GetDirToSave
   \param[in] pi - \~russian указатель на внутреннюю информацию.
   \~english pointer to internal information. \~
   */
DWRSSDK_EXTERN const char* DwRsSDK_GetDirToSave(dwrssdk_processing_info_t pi);
/**\copydoc DwRsSDK::IProcessingInfo::GetDataToSend
   \param[in] pi - \~russian указатель на внутреннюю информацию.
   \~english pointer to internal information. \~
   */
DWRSSDK_EXTERN const char* DwRsSDK_GetDataToSend(dwrssdk_processing_info_t pi);
/**\copydoc DwRsSDK::IProcessingInfo::GetSmtpAnswer
   \param[in] pi - \~russian указатель на внутреннюю информацию.
   \~english pointer to internal information. \~
   */
DWRSSDK_EXTERN const char* DwRsSDK_GetSmtpAnswer(dwrssdk_processing_info_t pi);
/**\copydoc DwRsSDK::IProcessingInfo::RemoveDirWithInfo
   \param[in] pi - \~russian указатель на внутреннюю информацию.
   \~english pointer to internal information. \~
   */
DWRSSDK_EXTERN int DwRsSDK_RemoveDirWithInfo(dwrssdk_processing_info_t pi);

/**\copydoc DwRsSDK::IProcessingInfo::GetScore
   \param[in] pi - \~russian указатель на внутреннюю информацию.
   \~english pointer to internal information. \~
   */
DWRSSDK_EXTERN int GetScore(dwrssdk_processing_info_t pi);

/** @} */

DWRSSDK_BEGIN

#ifdef __cplusplus

/**\~russian Объект, в котором хранится внутренняя информация при
   взаимодействии Receiver и drweb-maild.
\~english Object for storage of internal information during interaction of
    the Receiver and drweb-maild. \~
 */
class IProcessingInfo:virtual public IUnknown{
public:
 virtual ~IProcessingInfo(){}

 /**\~russian Если Receiver запущен с правами администратора (root),
    а drweb-maild - с правами другого пользователя, то необходимо установить
    требуемые права на файлы с сообщением перед передачей их на
    проверку drweb-maild.
    \~english If the Receiver is started with root privileges, and drweb-maild - with
    privileges of some other user, then it is necessary to set the required permissions
    to files with the message before these files are passed to drweb-maild for check. \~

    \param[in] uid - \~russian идентификатор пользователя, который надо установить на файлы.
    Укажите значение -1, если идентификатор менять не надо.
    \~english user identifier, which must be specified as files owner.
    Specify -1, if there is no need to change the identifier. \~
    \param[in] gid - \~russian идентификатор группы, который надо установить на файлы.
    Укажите значение -1, если идентификатор менять не надо.
    \~english group identifier which must be specified as files owner.
    Specify -1, if there is no need to change the identifier. \~
    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the action was successfully completed, 0 - an error occurred. \~

 */
 virtual bool        SetChown(uid_t uid,gid_t gid)=0;

 /**\~russian Возвращает строку с названием директории, в которой будет сохранено сообщение.
 \~english Returns the string with the name of the directory, to which the message will be saved. \~

    \return \~russian Строку с названием директории, в которой будет сохранено сообщение,
    либо пустую строку, если директория еще не была создана.
    \~english A string with the directory name, to which a message will be saved, or empty string,
    if the directory has not been created yet. \~
 */
 virtual const char* GetDirToSave()const=0;
 /**\~russian Возвращает строку с информацией, которая передается в drweb-maild (индекс запроса).
    Она позволяет drweb-maild однозначно определить директорию с сообщением (если используются
    одинаковые настройки).
    \~english Returns the string with information, which is passed to drweb-maild (request index).
    It allows drweb-maild to determine explicitly the directory with the message (if similar settings
    are used). \~

    \return \~russian Строку с информацией, которая передается в drweb-maild, либо пустую
    строку, если директория с сообщением еще не была создана.
     \~english A string with information, which is passed to drweb-maild, or empty string,
    if the directory has not been created yet. \~
 */
 virtual const char* GetDataToSend()const=0;
 /**\~russian Доступ к smtp ответу от drweb-maild.
 \~english Access to a smtp response from drweb-maild. \~

    \return \~russian Строку с полученным от drweb-maild smtp ответом, либо пустую строку,
    если такая информация не получена.
    \~english A string with the response received from drweb-maild smtp, or empty string,
    if such information was not received. \~
 */
 virtual const char* GetSmtpAnswer()const=0;

 /**\~russian Удаляет созданную директорию со всеми файлами в ней.
 \~english Deletes the created directory with all files in it. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \return \~english 1 - the action was successfully completed, 0 - an error occurred. \~
 */
 virtual int RemoveDirWithInfo()const=0;

    /**\~russian Доступ к счету для данного сообщения, который в итоге был
       установлен в drweb-maild \~english Access to score for this message,
       which was set in drweb-maild component \~

       \return \~russian счет для данного сообщения
       \~english score for this message \~
     */
    virtual int GetScore()const=0;
};

#endif

DWRSSDK_END

#endif
