/*--------------------------------------------------------------------------
 *  Copyright (c) 2006-2012 Igor Daniloff.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found in this directory.
 *------------------------------------------------------------------------*/

#ifndef DW_RS_SDK_IRECEIVER_H_
#define DW_RS_SDK_IRECEIVER_H_

#include "DwRsSDK_defs.H"
#include "DwRsSDK_IProcessingInfo.H"
#include "DwRsSDK_IComponentOptions.H"

/**\file
   \~russian Поддержка компонента Receiver.
   \~english Support of the Receiver component. \~
*/

/** \addtogroup C_INTERFACE
 *  @{
 */

/**\copydoc DwRsSDK::IReceiver
 */
typedef void* dwrssdk_receiver_t;
/**\copydoc DwRsSDK::CreateReceiver
 */
DWRSSDK_EXTERN dwrssdk_receiver_t DwRsSDK_CreateReceiver(void);
/**\copydoc DwRsSDK::DeleteReceiver
 */
DWRSSDK_EXTERN void           DwRsSDK_DeleteReceiver(dwrssdk_receiver_t r);
/**
   \~russian Инициализация объекта. Должна вызываться перед вызовом любой другой
    функции класса. Допускается повторный вызов, например, при перечитывании конфигурации.
    При вызове в другом потоке не должна вызываться ни одна функция класса.
    \~english Initialization of an object. Must be called before any other function of the class is called.
    A repeated call is allowed, for example, to re-read configuration. If it is called in another thread, no other
    function of the class should be called. \~

   \param[in] r - \~ \copydoc dwrssdk_receiver_t

   \param[in] base_dir - \~russian строка с адресом базовой директории Dr.Web MailD (параметр BaseDir из
    cекции General конфигурационного файла Dr.Web MailD). При установке Dr.Web MailD
    по умолчанию значение должно быть равно '/var/drweb/'.
    \~english a string with the path to Dr.Web MailD base directory (the BaseDir parameter from General
    section of Dr.Web MailD configuration file). If Dr.Web MailD was installed with default settings,
    the value is equal to '/var/drweb/'. \~

    \param[in] id - \~russian уникальный идентификатор Receiver`а. Директория с таким именем
    будет создаваться в /var/drweb/msgs/in (по умолчанию) с точкой в начале. Если существует
    Sender с таким же идентификатором, то отправляться письмо будет именно ему. В противном
    случае оно будет передаваться Sender`у по умолчанию.
    \~english unique identifier of the Receiver. It will be used in the name of the directory created
    in /var/drweb/msgs/in by default (with '.' symbol set in the beginning). If there exists a Sender
    with the same identifier, then a message will be passed to it. Otherwise it will be passed
    to the default Sender. \~

   \param[in] maild_options - \~russian логическое объединение одного из значений
    ::kMO_OnlyReceive, ::kMO_ReadyOnlyForBlock и ::kMO_ReadyForBlockAndSend.
    Любое из перечисленных значений может быть объединено с ::kMO_UseBackdoor,
    если Receiver реализует также и Callback сервис. При передаче значения ::kMO_ReadyForBlockAndSend
    допускается также объединение его либо со значением ::kMO_CanSendOnlyWithNewHeaders,
    либо с одним или несколькими значениями из набора ::kMO_CanNotEnvelopeModify,
    ::kMO_CanNotHeadersModify и ::kMO_CanNotBodyModify, в зависимости от того, какие
    модификации сообщения может осуществить компонент Receiver перед его отправкой.
    \~english logical combination of one of the values ::kMO_OnlyReceive, ::kMO_ReadyOnlyForBlock
    and ::kMO_ReadyForBlockAndSend.
    Any of the listed values can be combined with ::kMO_UseBackdoor, if the Receiver implements
    the Callback service as well. When the ::kMO_ReadyForBlockAndSend value is passed,
    it can be combined with either the ::kMO_CanSendOnlyWithNewHeaders value, or with
    one or several values from the set ::kMO_CanNotEnvelopeModify, ::kMO_CanNotHeadersModify
    and  ::kMO_CanNotBodyModify, depending on message modifications the Receiver component
    can make before the message is sent. \~

   \param[in] timeout - \~russian Таймаут (в секундах) на ожидание ответа от drweb-maild на одно сообщение.
    \~english timeout (in seconds) for a waiting of response from drweb-maild for one message. \~

   \param[in] min_num - \~russian минимальное число соединений, которое надо
    постоянно поддерживать с drweb-maild.
    \~english minimum number of connections with drweb-maild to be upholded permanently. \~

   \param[in] max_num - \~russian маскимальное число соединений, которое надо
    постоянно поддерживать с drweb-maild. Если значение равно нулю, то верхняя
    граница числа соединений не установлена. Если значение не равно 0, то оно
    должно быть >= чем значение min_num.
    \~english maximum number of connections with drweb-maild to be upholded permanently.
    If this value is equal to 0, then the upper limit of the amount of connections is not set.
    If this value is not equal to 0, then it must be >= than min_num value. \~

   \param[in] freetime - \~russian время в секундах, которое задает максимальный
    интервал бездействия соединения с drweb-maild. Если соединение не было ни
    разу использовано за это время, то оно будет закрыто. Данный параметр не
    влияет на первые min_num соединений - они будут всегда открыты.
    \~english time in seconds, defining maximum period of inactivity of connection with
    drweb-maild. If this connection was never used during this period, then it will be closed.
    This parameter does not affect the first min_num connections - they will be always open. \~

   \param[in] stat - \~russian вести ли статистику по динамике работы
   соединений. Набранную статистику можно будет потом сохранить в
   файл с помощью вызова DumpStat.
   \~english whether to collect statistics on dynamics of connections` operation.
    Collected statistics can be saved to the file with DumpStat call. \~

   \param[in] create_dir_struct - \~russian надо ли создавать структуру
   директорий для временных сообщений.
   \~english whether to create directory structure for temporary messages. \~

   \param[in] lock_id - \~russian надо блокировать запуск других Receiver`ов с
   тем же \p id.
   \~english startup of other Receivers with the same \p id must be blocked. \~

   \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
   \~english  1 - the task was successfully completed, 0 - an error occurred. \~
*/
DWRSSDK_EXTERN int DwRsSDK_InitializeReceiver(dwrssdk_receiver_t r,
                                              const char* base_dir,
                                              const char* id,
                                              int  maild_options,
                                              int  timeout,
                                              int  min_num,
                                              int  max_num,
                                              int  freetime,
                                              int  stat,
                                              int  create_dir_struct,
                                              int  lock_id);

/**
   \~russian Сохраняет статистику по использованию соединений в заданный файл.
   \~english Saves statistics on use of connections to the specified file. \~

   \param[in] r - \~ \copydoc dwrssdk_receiver_t

   \param[in] file - \~russian имя файла, в который должна быть сохранена статистика.
    \~english name of the file, to which statistics is saved. \~

   \return \~russian 1 - при успехе и 0 - при неудаче.
      \~english 1 - at success, 0 - at failure. \~
*/
DWRSSDK_EXTERN int DwRsSDK_DumpStat(dwrssdk_receiver_t r,const char* file);
/**\copydoc DwRsSDK::IReceiver::SetNewMsgDir

   \param[in] r - \~ \copydoc dwrssdk_receiver_t
 */
DWRSSDK_EXTERN dwrssdk_processing_info_t DwRsSDK_SetNewMsgDir(dwrssdk_receiver_t r);
/**\~russian Создает объект по обработке сообщения для уже существующей директории с сообщением.
    Это может быть полезно при восстановлении "потерянных" сообщений.
   \~english Creates a message processing object for an existing directory with the message.
     May be useful in recovery of "lost" messages.\~

   \param[in] r - \~ \copydoc dwrssdk_receiver_t

   \param[in] dir - \~russian директория с сообщением, строка должна заканчиваться знаком '/'.
   \~english a directory with the message, the string must end with '/' symbol. \~

   \return \~russian Объект с внутренней информацией по обработке сообщения, или NULL при ошибке.
   \~english Object with internal information on message processing, or NULL if an error occurs. \~

*/
DWRSSDK_EXTERN dwrssdk_processing_info_t DwRsSDK_GetProcessingInfoForDir(dwrssdk_receiver_t r,const char* dir);
/**
   \~russian Отправляет запрос на проверку сообщения модулю drweb-maild
    и получает ответ. Получение значения ::kACT_Error означает, что произошла
    внутренняя ошибка в Dr.Web MailD. ::kACT_Success означает, что drweb-maild принял
    сообщение на обработку и теперь полностью ответственен за него.
    Если у компонента Receiver доступен сервис Callback, то в дальнейшем возможен
    запрос к нему от drweb-maild на обработку этого сообщения.
    При получении значения ::kACT_Pass также могут быть установлены некоторые
    флаги из набора ::kACT_WasBodyModify, ::kACT_WasEnvelopeChange и
    ::kACT_WasTopHeaderModify, что потребует соответствующим образом обновить
    сообщение перед его отправкой.
    Значения ::kACT_Reject, ::kACT_Tempfail и ::kACT_Discard приводят к блокировке сообщения.
    \~english Sends a query to drweb-maild to check a message and receives a response.
    If the ::kACT_Error value is received, it means that an internal error in Dr.Web MailD occurred.
    If the ::kACT_Success value is received, it means that drweb-maild has received the message for
    processing and now is fully responsible for it.
    If the Callback service is available in the Receiver component, then drweb-maild may send a
    query to it to process this message.
    Upon the receipt of the kACT_Pass value some of the following flags can be set: kACT_WasBodyModify,
    kACT_WasEnvelopeChange and kACT_WasTopHeaderModify. This will require an update of the message
    before its dispatch. Receipt of kACT_Reject, kACT_Tempfail and kACT_Discard values results
    in message blocking. \~

    \param[in] r - \~ \copydoc dwrssdk_receiver_t

    \param[in] pi - \~russian объект с внутренней информацией по обработке сообщения.
    \~english object with internal information on message processing. \~

    \param[in] co - \~russian дополнительные параметры, которые надо передать
    вместе с сообщением. Значение может быть равным 0, если не надо
    передавать никаких дополнительных параметров.
    \~english additional parameters to be passed with the message. Value may be equal to 0, if
    no additional parameters will be passed. \~

    \param[in] score - \~russian начальный счет для данного сообщения. 0 -
                       счет поу-молчанию.
     \~english start score for this message. 0 - default score \~

    \param[in] maild_options - \~russian можно передать специфичные флаги для запроса в майлд. \~
    \param[in] timeout - \~russian можно передать специфичный таймаут для сессии с майлд. \~

    \return \~russian Флаги (или их объединение) из ::DwAction.
    \~english Flags (or their combination) from ::DwAction. \~

 */
DWRSSDK_EXTERN int DwRsSDK_SendRequestToMaild( dwrssdk_receiver_t r,
    dwrssdk_processing_info_t pi, dwrssdk_component_options_t co, int score, int maild_options, int timeout);

/** \~russian Попытка восстановления сообщений из директории Receiver`а.
     \~english An attempt to restore messages from Receiver directory. \~

 \param[in] r - \~ \copydoc dwrssdk_receiver_t

 @param[in] check_all - \~russian восстанавливать ли все сообщения или только те, в
 директории которых содержится файл об ошибке ресивера
  \~english restore all messages or only those, which directory contains a receiver error file.  \~


 @return \~russian 1 - при успехе и 0 - при неудаче.
      \~english 1 - at success, 0 - at failure. \~
*/
DWRSSDK_EXTERN int DwRsSDK_ReceiverRestoreMsgs(dwrssdk_receiver_t r,int check_all);

/**
 \copydoc DwRsSDK::IReceiver::GetBaseDir

 @param[in] r - \~ \copydoc dwrssdk_receiver_t
*/
DWRSSDK_EXTERN const char* DwRsSDK_ReceiverGetBaseDir(dwrssdk_receiver_t r);

/**\~russian Освобождает память выделенную через ::DwRsSDK_SetNewMsgDir
    или ::DwRsSDK_GetProcessingInfoForDir.
   \~english Deallocates memory allocated via ::DwRsSDK_SetNewMsgDir
    or ::DwRsSDK_GetProcessingInfoForDir. \~

   \param[in] pi - \~russian объект с внутренней информацией по обработке сообщения.
   \~english object with internal information on the message processing. \~
*/
DWRSSDK_EXTERN void DwRsSDK_DeleteProcessingInfo(dwrssdk_processing_info_t pi);

/** @} */

DWRSSDK_BEGIN

#ifdef __cplusplus

/**\~russian Реализация поддержки компонента Receiver.
\~english Implementation of the Receiver component support. \~
 */
class IReceiver:virtual public IUnknown
{
public:
        virtual ~IReceiver(){};

    /**\~russian Инициализация объекта. Должна вызываться перед вызовом любой другой
    функции класса. Допускается повторный вызов, например, при перечитывании конфигурации.
    При вызове в другом потоке не должна вызываться ни одна функция класса.
    \~english Initialization of an object. Must be called before any other function of the class is called.
    A repeated call is allowed, for example, to re-read configuration. If it is called in another thread, no other
    function of the class should be called. \~

    \param[in] base_dir - \~russian строка с адресом базовой директории Dr.Web MailD (параметр BaseDir из
    cекции General конфигурационного файла Dr.Web MailD). При установке Dr.Web MailD
    по умолчанию значение должно быть равно '/var/drweb/'.
    \~english a string with the path to Dr.Web MailD base directory (the BaseDir parameter from General
    section of Dr.Web MailD configuration file). If Dr.Web MailD was installed with default settings,
    the value is equal to '/var/drweb/'. \~

    \param[in] id - \~russian уникальный идентификатор Receiver`а. Директория с таким именем
    будет создаваться в /var/drweb/msgs/in (по умолчанию) с точкой в начале. Если существует
    Sender с таким же идентификатором, то отправляться письмо будет именно ему. В противном
    случае оно будет передаваться Sender`у по умолчанию.
    \~english unique identifier of the Receiver. It will be used in the name of the directory created
    in /var/drweb/msgs/in by default (with '.' symbol set in the beginning). If there exists a Sender
    with the same identifier, then a message will be passed to it. Otherwise it will be passed
    to the default Sender. \~

    \param[in] maild_options - \~russian логическое объединение одного из значений
    ::kMO_OnlyReceive, ::kMO_ReadyOnlyForBlock и ::kMO_ReadyForBlockAndSend.
    Любое из перечисленных значений может быть объединено с ::kMO_UseBackdoor,
    если Receiver реализует также и Callback сервис. При передаче значения ::kMO_ReadyForBlockAndSend
    допускается также объединение его либо со значением ::kMO_CanSendOnlyWithNewHeaders,
    либо с одним или несколькими значениями из набора ::kMO_CanNotEnvelopeModify,
    ::kMO_CanNotHeadersModify и ::kMO_CanNotBodyModify, в зависимости от того, какие
    модификации сообщения может осуществить компонент Receiver перед его отправкой.
    \~english logical combination of one of the values ::kMO_OnlyReceive, ::kMO_ReadyOnlyForBlock
    and ::kMO_ReadyForBlockAndSend.
    Any of the listed values can be combined with ::kMO_UseBackdoor, if the Receiver implements
    the Callback service as well. When the ::kMO_ReadyForBlockAndSend value is passed,
    it can be combined with either the ::kMO_CanSendOnlyWithNewHeaders value, or with
    one or several values from the set ::kMO_CanNotEnvelopeModify, ::kMO_CanNotHeadersModify
    and  ::kMO_CanNotBodyModify, depending on message modifications the Receiver component
    can make before the message is sent. \~

    \param[in] timeout - \~russian Таймаут (в секундах) на ожидание ответа от drweb-maild на одно сообщение.
    \~english timeout (in seconds) for a waiting of response from drweb-maild for one message. \~

    \param[in] min_num - \~russian минимальное число соединений, которое надо
    постоянно поддерживать с drweb-maild.
    \~english minimum number of connections with drweb-maild to be upholded permanently. \~

    \param[in] max_num - \~russian маскимальное число соединений, которое надо
    постоянно поддерживать с drweb-maild. Если значение равно нулю, то верхняя
    граница числа соединений не установлена. Если значение не равно 0, то оно
    должно быть >= чем значение min_num.
    \~english maximum number of connections with drweb-maild to be upholded permanently.
    If this value is equal to 0, then the upper limit of the amount of connections is not set.
    If this value is not equal to 0, then it must be >= than min_num value. \~

    \param[in] freetime - \~russian время в секундах, которое задает максимальный
    интервал бездействия соединения с drweb-maild. Если соединение не было ни
    разу использовано за это время, то оно будет закрыто. Данный параметр не
    влияет на первые min_num соединений - они будут всегда открыты.
    \~english time in seconds, defining maximum period of inactivity of connection with
    drweb-maild. If this connection was never used during this period, then it will be closed.
    This parameter does not affect the first min_num connections - they will be always open.\~

    \param[in] stat - \~russian вести ли статистику по динамике работы соединений.
    Набранную статистику можно будет потом сохранить в файл с помощью вызова DumpStat.
    \~english whether to collect statistics on dynamics of connections` operation.
    Collected statistics can be saved to the file with DumpStat call. \~

   \param[in] create_dir_struct - \~russian надо ли создавать структуру директорий
   для временных сообщений.
     \~english whether to create directory structure for temporary messages.\~

   \param[in] lock_id - \~russian надо блокировать запуск других Receiver`ов с
   тем же \p id.
    \~english startup of other Receivers with the same \p id must be blocked. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
   \~english  1 - the task was successfully completed, 0 - an error occurred. \~
 */
 virtual bool Initialize(const char* base_dir,
                         const char* id,
                         int  maild_options,
                         int  timeout,
                         int  min_num,
                         int  max_num,
                         int  freetime,
                         bool stat,
                         bool create_dir_struct,
                         bool lock_id
                         )=0;

    /**\~russian Cохраняет статистику по использованию соединений в заданный файл.
      \~english Saves statistics on use of connections to the specified file. \~

       \param[in] file - \~russian имя файла, в который должна быть сохранена статистика.
      \~english name of the file, to which statistics is saved. \~

       \return \~russian 1- при успехе и 0 - при неудаче.
      \~english 1 - at success, 0 - at failure. \~
     */
    virtual bool DumpStat(const char* file)=0;

 /**\~russian Создает новую директорию для сообщения. После успешного выполнения функции,
    Receiver становится ответственным за удаление директории со всем содержимым.
    Если drweb-maild после проверки сообщения вернул значение, отличное от ::kACT_Pass или
    ::kACT_Error, то ответственность за директорию переходит к drweb-maild, а компонент
    Receiver должен "забыть" об этой директории.
    \~english Creates a new directory for a message. After the function is successfully performed,
    the Receiver component becomes responsible for deletion of the directory and its contents.
    If drweb-maild checked the message and returned the value different from  ::kACT_Pass or
    ::kACT_Error, then drweb-maild becomes responsible for the directory, and the Receiver component
    must "forget" about this directory. \~

           \return  \~russian Объект с внутренней информацией по обработке сообщения, или NULL при ошибке.
    \~english Object with internal information on message processing, or NULL if an error occurs. \~
 */
 virtual IProcessingInfo* SetNewMsgDir()const=0;
 /**\~russian Создает объект по обработке сообщения для уже существующей директории
    с сообщением. Это может быть полезно при восстановлении "потерянных" сообщений.
    \~english Creates a message processing object for an existing directory with the message.
     May be useful in recovery of "lost" messages.\~

    \param[in] dir - \~russian Директория с сообщением, строка должна заканчиваться знаком '/'.
    \~english Directory with the message, the string must end with '/' symbol. \~

    \return \~russian Объект с внутренней информацией по обработке сообщения, или NULL при ошибке.
    \~english Object with internal information on message processing, or NULL if an error occurs. \~
  */
 virtual IProcessingInfo* GetProcessingInfoForDir(const char* dir)const=0;

 /**\~russian Отправляет запрос на проверку сообщения модулю drweb-maild
    и получает ответ. Получение значения ::kACT_Error означает, что произошла
    внутренняя ошибка в Dr.Web MailD. ::kACT_Success означает, что drweb-maild принял
    сообщение на обработку и теперь полностью ответственен за него.
    Если у компонента Receiver доступен сервис Callback, то в дальнейшем возможен
    запрос к нему от drweb-maild на обработку этого сообщения.
    При получении значения ::kACT_Pass также могут быть установлены некоторые
    флаги из набора ::kACT_WasBodyModify, ::kACT_WasEnvelopeChange и
    ::kACT_WasTopHeaderModify, что потребует соответствующим образом обновить
    сообщение перед его отправкой.
    Значения ::kACT_Reject, ::kACT_Tempfail и ::kACT_Discard приводят к блокировке сообщения.
    \~english Sends a query to drweb-maild to check a message and receives a response.
    If the ::kACT_Error value is received, it means that an internal error in Dr.Web MailD occurred.
    If the ::kACT_Success value is received, it means that drweb-maild has received the message for
    processing and now is fully responsible for it.
    If the Callback service is available in the Receiver component, then drweb-maild may send a
    query to it to process this message.
    Upon the receipt of the kACT_Pass value some of the following flags can be set: kACT_WasBodyModify,
    kACT_WasEnvelopeChange and kACT_WasTopHeaderModify. This will require an update of the message
    before its dispatch. Receipt of kACT_Reject, kACT_Tempfail and kACT_Discard values results
    in message blocking. \~

    \param[in] pi - \~russian объект с внутренней информацией по обработке сообщения.
    \~english an object with internal information on message processing. \~

    \param[in] co - \~russian дополнительные параметры, которые надо передать
    вместе с сообщением. Значение может быть равным 0 - если не надо
    передавать никаких дополнительных параметров.
    \~english additional parameters to be passed with the message. Value may be equal to 0, if
    no additional parameters will be passed. \~

    \param[in] score - \~russian начальный счет для данного сообщения. 0 -
          счет поу-молчанию.
     \~english start score for this message. 0 - default score \~

     \param[in] maild_options - \~russian можно задать кастомные флаги для майлд \~
     \param[in] timeout - \~russian можно задать кастомный таймаут на сессию с майлд \~

    \return \~russian Флаги (или их объединение) из ::DwAction.
    \~english Flags (or their combination) from ::DwAction. \~
  */
    virtual int SendRequestToMaild(IProcessingInfo* pi,IComponentOptions* co,
        int score = 0, int maild_options=0, int timeout=0 )const=0;

    /** \~russian Попытка восстановления сообщений из директории Receiver`а.
    \~english An attempt to restore messages from the Receiver directory. \~

     @param[in] check_all - \~russian восстанавливать все сообщения или только те, в
     директории которых содержится файл о ошибке ресивера
     \~english restore all messages or only those, which directory contains a receiver error file. \~

     @return \~russian True - при успехе и false - при неудаче.
    \~english True - at success, false - at failure. \~
    */
    virtual bool RestoreMsgs(bool check_all)=0;

 /**\~russian Освобождает память, выделенную через
    IReceiver::SetNewMsgDir или IReceiver::GetProcessingInfoForDir.
    \~english Deallocates memory allocated via
    IReceiver::SetNewMsgDir or IReceiver::GetProcessingInfoForDir. \~

    \param[in] pi - \~russian Объект с внутренней информацией по обработке сообщения.
    \~english Object with internal information on message processing. \~
 */
    static void DeleteProcessingInfo(IProcessingInfo* pi);

    /** \~russian Доступ к базовой директории Receiver`а. Право писать туда
     имеет только данный компонент.
       \~english An access to the base directory of the Receiver component. A write access to it
     is provided to the given component only.\~


     @return \~russian Абсолютный путь к директории. Память под строку удалять нельзя.
     \~english Absolute path to the directory. A memory allocated for a string must be preserved.\~
    */
    virtual const char* GetBaseDir()const =0;
};

/**\~russian Создает новый объект поддержки компонента Receiver.
\~english Creates a new Receiver component support object. \~
   \return \~russian Cозданный объект или NULL при ошибке.
   \~english Created object or NULL, if an error occurs. \~
*/
DWRSSDK_EXTERN IReceiver* CreateReceiver();
/**\~russian Освобождает память объекта поддержки компонента Receiver.
\~english Deallocates memory of the Receiver component support object. \~

   \param[in] r - \~russian указатель на объект, который нужно освободить.
   \~english pointer to the object to be deallocated. \~
*/
DWRSSDK_EXTERN void       DeleteReceiver(IReceiver* r);

#endif

DWRSSDK_END

#endif
