/*--------------------------------------------------------------------------
 *  Copyright (c) 2006-2012 Igor Daniloff.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found in this directory.
 *------------------------------------------------------------------------*/

#ifndef DW_RS_SDK_ISENDER_H_
#define DW_RS_SDK_ISENDER_H_

#include "DwRsSDK_IServer.H"
#include "DwRsSDK_IComponentOptions.H"

/**\file
   \~russian Поддержка компонента Sender.
   \~english Support of the Sender component. \~
*/

/** \addtogroup C_INTERFACE
 *  @{
 */

/**\copydoc DwRsSDK::ISender
 */
typedef void* dwrssdk_sender_t;


/** \~russian Объявление функции обратного вызова для обработки запросов в
     компоненте Sender.
 \~english Declaration of the callback function for processing requests in the Sender component. \~

    \param[in] dir - \~russian строка с адресом директории, в которой находится сообщение.
    \~english a string with the path to directory where the message resides. \~

    \param[in] orig_data - \~russian строка с исходным запросом на обработку
    (из нее была получена \a dir).
    \~english a string with the initial processing request (from it \a dir was received). \~

    \param[in] comp_opt - \~russian параметры компонентов, полученные вместе с письмом.
    Значение выделяется в куче, поэтому может быть сохранено в DwPtr для сохранения.
    \~english components' parameters, received with the message. Value is allocated in heap,
    and can be saved in DwPtr for preservation.  \~

    \param[in] need_remove - \~russian 1 - если директорию с сообщением \a dir требуется
     удалить после обработки (если возвращается ::kACT_Pass), и 0 - в противном случае.
     \~english 1 - if the \a dir directory with the message must be deleted after the processing
     (if ::kACT_Pass is returned), otherwise - 0. \~

    \param[in] options - \~russian параметры полученные от клиента. Возможные значения
     определены в ::eSenderOptions.
     \~english parameters received from the client. Possible values are specified in ::eSenderOptions. \~

    \param[in] timeout - \~russian таймаут на обработку (в секундах), заданный клиентом.
    Если значение равно 0, то таймаут не задан.
   \~english timeout (in seconds) for processing, specified by the client. If value is set to 0,
    then the timeout is not specified. \~

    \return \~russian ::kACT_Pass при успешной обработке и ::kACT_Reject при ошибке.
     \~english ::kACT_Pass, if the processing was successful, and ::kACT_Reject, if an error occurred. \~
    */
typedef int (*dwrssdk_sender_processing_t)(const char* dir,
                                           const char* orig_data,
#ifdef __cplusplus
                                           DWRSSDK_NAMESPACE::IComponentOptions*
#else
                                           dwrssdk_component_options_t
#endif
                                           comp_opt,
                                           int need_remove,
                                           int  options,
                                           int  timeout);


/** \~russian Имя файла, который можно создать в директории сообщения, чтобы
 вызов DwRsSDK::ISender::RestoreMsgs (DwRsSDK_SenderRestoreMsgs) с параметром
 false (0) также обработал и это сообщение.
   \~english Name of the file to be created in the message directory, to enable processing of this message
by DwRsSDK::ISender::RestoreMsgs (DwRsSDK_SenderRestoreMsgs) call with false (0) parameter.  \~
*/
//DWRSSDK_EXTERN const char* dwrssdk_sender_resend_filename;

/* \~russian Oбъявление функции обратного вызова для восстановления потерянных
 сообщений.
   \~english Declaration of the callback function for recovery of lost messages.\~

 \param[in] path - \~russian абсолютный путь к директории с сообщением.
    \~english absolute path to the message directory. \~

 \param[in] data - \~russian дополнительная информация, передаваемая с каждым вызовом.
    \~english additional information, passed with each call. \~

 \return \~russian 1 - при успехе, и 0 - если была ошибка, и директорию \p path надо удалить.
    \~english 1 - at success, 0 - if some error occurs, and \p path directory must be deleted. \~
*/
typedef int (*dwrssdk_sender_restore_func_t)(const char* path,void* data);

/**\~russian Регистрация функции обратного вызова для обработки
   запросов от клиентов. Должна вызываться перед любым вызовом
   функции обработки запросов клиента.
   \~english Registration of the callback function for processing queries from
   clients. Must be called before any call of the client's query processing function. \~
   \sa DwRsSDK::IServer::Process DwRsSDK_ProcessCallback DwRsSDK_ProcessSender

   \param[in] s - \~ \copydoc dwrssdk_sender_t

   \param[in] cp - \~russian Указатель на функцию для обработки.
   \~english Pointer to the function for processing. \~
*/
DWRSSDK_EXTERN int DwRsSDK_RegisterProcessingFuncSender(dwrssdk_sender_t s,dwrssdk_sender_processing_t cp);

/**\copydoc DwRsSDK::CreateSender
 */
DWRSSDK_EXTERN dwrssdk_sender_t       DwRsSDK_CreateSender(void);
/**\copydoc DwRsSDK::DeleteSender
 */
DWRSSDK_EXTERN void       DwRsSDK_DeleteSender(dwrssdk_sender_t r);
/**
   \~russian Инициализация настройки сервера. Должна вызываться
    перед любой функцией данного класса и функциями его производных
    классов. Допускается повторный вызов (к примеру, при перечитывании
    конфигурации), но при этом все потоки, вызывающие IServer::Process,
    должны быть заблокированы через IServer::Block.
    После успешной инициализации потоки следует снова разблокировать
    через IServer::Unblock.
    \~english Initialization of the server settings. Must be called before any function
    of the given class and functions of its derived classes. Repeated call is allowed
    (for example, to re-read the configuration), but all the threads calling for IServer::Process
    must be blocked via IServer::Block. After successful initialization these threads
    should be unblocked via IServer::Unblock. \~

   \param[in] s - \~ \copydoc dwrssdk_sender_t

   \param[in] id - \~russian уникальный идентификатор Sender`а. Директория с
   таким именем будет создаваться в /var/drweb/msgs/out (по умолчанию) с точкой в начале.
   Также будет создан сокет в /var/drweb/ipc (по умолчанию), с именем .id.
   \~english unique identifier of the Sender. It will be used in the name of the directory created
    in /var/drweb/msgs/out by default (with '.' symbol set in the beginning). Also a socket
    will be created in /var/drweb/ipc by default, with .id as a part of its name. \~

   \param[in] base_dir - \~russian строка с адресом базовой директории Dr.Web MailD (параметр BaseDir из
    cекции General конфигурационного файла Dr.Web MailD). Должна заканчиваться знаком '/'.
    При установке Dr.Web MailD по умолчанию значение должно быть равно '/var/drweb/'.
    \~english a string with the path to Dr.Web MailD base directory (the BaseDir parameter from General
    section of Dr.Web MailD configuration file).  Must end with '/' symbol. If Dr.Web MailD was installed
    with default settings, the value is equal to '/var/drweb/'. \~

   \param[in] timeout - \~russian значение таймаута (в секундах) на
    работу с одним клиентом.
    \~english value of the timeout (in seconds) for operation with one client. \~

   \param[in] stat - \~russian собирать ли статистику.
     \~english  whether to collect statistics. \~

   \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
   \~english  1 - the task was successfully completed, 0 - an error occurred. \~

*/
DWRSSDK_EXTERN int DwRsSDK_InitializeSender(dwrssdk_sender_t s,
                                            const char* base_dir,
                                            const char* id,
                                            int  timeout,
                                            int  stat
                                            );
/**
   \~russian Сохраняет статистику по использованию соединений в
       заданный файл.
    \~english Collects statistics on the use of connections to the specified file.\~

   \param[in] r - \~ \copydoc dwrssdk_sender_t
   \param[in] file - \~russian имя файла, в который должна быть сохранена статистика.
    \~english name of the file to which statistics is saved. \~

   \return \~russian 1 - при успехе и 0 - при неудаче.
        \~english 1 - at success, 0 - at failure. \~

*/
DWRSSDK_EXTERN int DwRsSDK_DumpStatSender(dwrssdk_sender_t r,const char* file);

/**
   \~russian Функция блокирует все потоки работающие в IServer::Process до
     вызова IServer::Unblock.
   \~english The function blocks all threads working in IServer::Process
     until IServer::Unblock is called. \~

   \param[in] s - \~ \copydoc dwrssdk_sender_t

   \param[in] timeout - \~russian таймаут (в секундах) на выполнение блокировки.
   \~english timeout (in seconds) for blocking. \~

   \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
   \~english  1 - the task was successfully completed, 0 - an error occurred. \~

*/
DWRSSDK_EXTERN int DwRsSDK_BlockSender(dwrssdk_sender_t s,int timeout);
/**\copydoc DwRsSDK::IServer::Unblock

   \param[in] s - \~ \copydoc dwrssdk_sender_t
 */
DWRSSDK_EXTERN void DwRsSDK_UnblockSender(dwrssdk_sender_t s);
/**
   \~russian Посылает запрос на остановку потока, работающего в IServer::Process.
      Данный поток выходит из IServer::Process с кодом ::PR_StopThread. Эта функция
      не ждет завершения работы потока - такую проверку должно выполнить само приложение.
   \~english Sends a query to stop a thread working in IServer::Process.
      This thread exits the IServer::Process with the PR_StopThread code. This function does not wait for the
      thread to terminate its operation - the application must check this itself. \~

   \param[in] s - \~ \copydoc dwrssdk_sender_t

   \param[in] tid - \~russian идентификатор потока для остановки.
   \~english identifier of a thread for termination. \~
*/
DWRSSDK_EXTERN void DwRsSDK_StopThreadSender(dwrssdk_sender_t s,const void* tid);

/**
   \~russian Обработка запросов клиента. Должна вызываться в бесконечном цикле.
       При завершении с кодом ::PR_StopThread приложение должно выйти из цикла и
       завершить выполнение потока.
       \~english Processing of client queries. Must be called in an infinite loop. If ended with
       the ::PR_StopThread code, the application must exit the loop and terminate execution
       of the thread. \~

   \param[in] s - \~ \copydoc dwrssdk_sender_t

   \return \~russian Код завершения обработки.
   \~english Processing termination code. \~
*/
DWRSSDK_EXTERN DwRsSDK_ProcessRes DwRsSDK_ProcessSender(dwrssdk_sender_t s);

/**
   \~russian Обработка запросов клиента. Должна вызываться в бесконечном цикле.
       При завершении с кодом ::PR_StopThread приложение должно выйти из цикла и
       завершить выполнение потока.
       \~english Processing of client queries. Must be called in an infinite loop. If ended with
       the ::PR_StopThread code, the application must exit the loop and terminate execution
       of the thread. \~

   \param[in] s - \~ \copydoc dwrssdk_sender_t

   \param[in] timeout - \~russian таймаут, в течение которого нужно ожидать запросов
       от клиентов. 0 - не устанавливать таймаут. Если произошел таймаут, то будет возвращено
       ::PR_StopThread.
       \~english timeout for awaiting requests from clients. 0 - timeout is not set. If timeout is expired, then
       ::PR_StopThread is returned. \~

   \return \~russian Код завершения обработки.
   \~english Processing termination code. \~
 */
DWRSSDK_EXTERN DwRsSDK_ProcessRes DwRsSDK_ProcessSenderTimeout(dwrssdk_sender_t s,int timeout);

/** \~russian Попытка восстановления сообщений из директории Sender`а.
     \~english An attempt to restore messages from Sender directory. \~

 @param[in] s - \~ \copydoc dwrssdk_sender_t

 @param[in] check_all - \~russian восстанавливать все сообщения или только те,
     в директории которых содержится файл, имя которого содержится в \sa
     dwrssdk_sender_resend_filename.
     \~english restore all messages or only those, which directory contains a file with the
     name from \sa dwrssdk_sender_resend_filename. \~

 @param[in] func - \~russian функция обратного вызова, которая будет
     вызываться для каждого найденного сообщения. В качестве параметра
     функции будет передаваться абсолютный путь к директории с сообщением и
     значение параметра \p data. Если функция вернет 1, то предполагается, что
     Sender принял сообщения, если же вернется 0, то предполагается ошибка,
     и эта директория будет удалена.
     \~english a callback function to be called for every message found. As a parameter of this function
     an absolute path to the message directory and value of \p data parameter will be passed.
     If the function returns 1, then it is supposed, that Sender has accepted the message.
     If the function returns 0, then it is supposed, that an error has occurred, and specified directory
     will be deleted.\~

 @param[in] data - \~russian дополнительная информация, которую надо
  передать в каждый вызов \p func.
  \~english additional information to be passed to every \p func call. \~

 @return \~russian True - при успехе и false - при неудаче.
      \~english True - at success, false - at failure. \~
*/
DWRSSDK_EXTERN int DwRsSDK_SenderRestoreMsgs(dwrssdk_sender_t s,int check_all,dwrssdk_sender_restore_func_t func,void* data);

/**
 \copydoc DwRsSDK::ISender::GetBaseDir

 @param[in] s - \~ \copydoc dwrssdk_sender_t
*/
DWRSSDK_EXTERN const char* DwRsSDK_SenderGetBaseDir(dwrssdk_sender_t s);

/** @} */

DWRSSDK_BEGIN

/**\~russian Функция удаляет информацию о сообщении. Вызывается из обработчика запросов от клиента.
\~english The function deletes information on the message. It is called from the query handler from the client. \~

   \param[in] dir - \~russian директория с сообщением. \sa dwrssdk_sender_processing_t
   \~english directory with the message. \sa dwrssdk_sender_processing_t \~

   \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
   \~english  1 - the task was successfully completed, 0 - an error occurred. \~
 */
DWRSSDK_EXTERN int RemoveDirWithInfo(const char* dir);

#ifdef __cplusplus

class DwSenderFunctor;

/**\~russian Реализация поддержки компонента Sender.
\~english Implementation of the Sender component support. \~
 */
class ISender:virtual public IServer
{
public:
    virtual ~ISender(){}

    /**\copydoc DwRsSDK::ICallback::RegisterProcessingFunc(dwrssdk_callback_processing_t)
     */
    virtual bool RegisterProcessingFunc(dwrssdk_sender_processing_t cp)=0;
    /**\copydoc DwRsSDK::ICallback::RegisterProcessingFunc(DwCallbackFunctor*)
     */
    virtual bool RegisterProcessingFunc(DwSenderFunctor* cp)=0;

    /** \~russian Попытка восстановления сообщений из директории Sender`а.
       \~english An attempt to restore messages from Sender directory. \~

     @param[in] check_all - \~russian восстанавливать все сообщения или только те,
     в директории которых содержится файл, имя которого содержится в \sa
     dwrssdk_sender_resend_filename.
     \~english restore all messages or only those, which directory contains a file with the
     name from \sa dwrssdk_sender_resend_filename. \~

     @param[in] func - \~russian функция обратного вызова, которая будет
     вызываться для каждого найденного сообщения. В качестве параметра
     функции будет передаваться абсолютный путь к директории с сообщением и
     значение параметра \p data. Если функция вернет 1, то предполагается, что
     Sender принял сообщения, если же вернется 0, то предполагается ошибка,
     и эта директория будет удалена.
     \~english a callback function to be called for every message found. As a parameter of this function
     an absolute path to the message directory and value of \p data parameter will be passed.
     If the function returns 1, then it is supposed, that Sender has accepted the message.
     If the function returns 0, then it is supposed, that an error has occurred, and specified directory
     will be deleted.\~

     @param[in] data - \~russian дополнительная информация, которую надо передать
     в каждый вызов \p func.
     \~english additional information to be passed to every \p func call. \~

     @return \~russian True - при успехе и false - при неудаче.
    \~english True - at success, false - at failure. \~
    */
    virtual bool RestoreMsgs(bool check_all,dwrssdk_sender_restore_func_t func,void* data)=0;

    /** \~russian Доступ к базовой директории Sender`а. Право писать туда
     имеет только данный компонент.
       \~english An access to the base directory of the Sender component. A write access to it
     is provided to the given component only.\~


     @return \~russian Aбсолютный путь к директории. Память под строку удалять нельзя.
     \~english Absolute path to the directory. A memory allocated for a string must be preserved.\~
    */
    virtual const char* GetBaseDir()const =0;

};

/**\~russian Функтор для обработки запросов от клиентов для компонента Sender.
\~english Functor for processing queries from clients for the Sender component.
 */
class DwSenderFunctor {
public:
    /**\copydoc DwRsSDK::DwCallbackFunctor::DwCallbackFunctor
     */
    DwSenderFunctor(dwrssdk_sender_processing_t t=0):
        m_caller(t)
    {}

    /**\~russian Прокси-вызов обработчика.
       \sa dwrssdk_sender_processing_t
       \~english Proxy call for the handler.
       \sa dwrssdk_sender_processing_t \~
    */
    virtual int operator()(const char* dir,
                           const char* orig_data,
                           IComponentOptions* comp_opt,
                           bool need_remove,
                           int  options,
                           int  timeout)
    {
        if(m_caller)
            return m_caller(dir,orig_data,comp_opt,need_remove,options,timeout);
        else
            return kACT_None;
    }

    virtual ~DwSenderFunctor() {}
private:
    dwrssdk_sender_processing_t m_caller;
};

/**\~russian Функция создает новый объект поддержки компонента Sender.
\~english The function creates a new Sender component support object. \~
   \return \~russian Созданный объект, или NULL при ошибке.
   \~english Created object, or NULL if an error occurs. \~
*/
DWRSSDK_EXTERN ISender*   CreateSender();
/**\~russian Функция освобождает память объекта поддержки компонента Sender.
\~english The function deallocates memory of the Sender component support object. \~

   \param[in] r - \~russian указатель на объект для освобождения.
   \~english pointer to the object to be deallocated. \~
*/
DWRSSDK_EXTERN void       DeleteSender(ISender* r);

#endif

DWRSSDK_END

#endif
