/*--------------------------------------------------------------------------
 *  Copyright (c) 2006-2012 Igor Daniloff.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found in this directory.
 *------------------------------------------------------------------------*/

#ifndef DW_RS_SDK_LOG_H_
#define DW_RS_SDK_LOG_H_

#include <syslog.h>

#include "DwRsSDK_defs.H"

/**\file
   \~russian Поддержка системы ведения лога для SDK.
      \~english Support of logging for SDK. \~
*/

DWRSSDK_BEGIN

/** \addtogroup C_INTERFACE
 *  @{
 */

/**\~russian Структура с инициализационными данными лога. Для инициализации лога
    заполненную структуру следует передать функции \ref InitLog .
   \~english Structure with data which initializes the log. For initialization od the log the filled structure
    should be passed to \ref InitLog function. \~
   */
typedef struct LogInfoType
{
    const char*  m_name;/*!<\~russian Имя программы, которая выводит информацию в лог.
   \~english Name of the logging program. \~*/

    const char*  m_log_level;/*!<\~russian Уровень подробности лога. Строка регистронезависимая.
    Возможные значения: Quiet, Error, Alert, Info, Debug.
    \~english Log verbosity level. The string is case-insensitive. Possible values: Quiet, Error, Alert, Info, Debug. \~*/
    const char*  m_syslog_facility;/*!<\~russian Служба вывода в syslog. Строка регистронезависимая.
    Возможные значения: Daemon, Mail, Local[0-7].
    \~english Output to syslog. The string is case-insensitive. Possible values: Daemon, Mail, Local[0-7]. \~*/
    int          m_print_to_console;/*!<\~russian Требуется ли выводить сообщения также и в консоль.
    1 - выводить в консоль, 0 - не выводить в консоль.
    \~english Whether messages should also be output to console. 1 - output to console, 0 - do not output to console. \~*/
    const char*  m_ipc_log_level;/*!<\~russian Уровень лога межпроцессного взаимодействия.
    Строка регистронезависимая. Возможные значения: Quiet, Error, Alert, Info, Debug.
    \~english IPC interaction log level. The string is case-insensitive.
    Possible values: Quiet, Error, Alert, Info, Debug. \~*/
    const char*  m_layout;/*!<\~russian В данной версии SDK не используется.
    \~english Not used in current version of SDK. \~*/
    const char*  m_filename;/*!<\~russian Имя файла, в который будет сохраняться лог, или
    "syslog", если лог будет выводиться через syslog.\~english FIXME */
} LogInfo;

/**\~russian Функция инициализирует систему логов в DwRsSDK.
    \~english The function initializes the system of logs in DwRsSDK. \~
   \param[in] log - \~russian указатель на заполненную структуру \ref LogInfoType.
   \~english pointer to the filled \ref LogInfoType structure. \~
   \return \~russian 0 - действие успешно выполнено, 1 - произошла ошибка.
   \~english 0 - the task was successfully completed, 1 - an error occurred. \~
*/
DWRSSDK_EXTERN int InitLog(const LogInfo* log);

/**\~russian Вывод в лог средствами SDK.
\~english Output to log by means of SDK. \~

   \param[in] level - \~russian Уровень подробности лога. Совпадает со значениями уровня
   подробности лога для syslog (man 3 syslog).
   \~english Log verbosity level. It must be the same as the log level values for syslog (man 3 syslog). \~
   \param[in] str - \~russian Строка для вывода.
   \~english String for output. \~
 */
DWRSSDK_EXTERN void DwLog(int level,const char* str,...);

/**\~russian Вывод в лог средствами SDK.
   \~english Output to log by means of SDK. \~

   \param[in] level - \~russian Уровень подробности лога. Строка регистронезависимая.
    Возможные значения: Quiet, Error, Alert, Info, Debug.
    \~english Log verbosity level. The string is case-insensitive. Possible values: Quiet, Error, Alert, Info, Debug. \~
   \param[in] str - \~russian Строка для вывода.
   \~english String for output. \~
 */
DWRSSDK_EXTERN void DwLog2(const char* level,const char* str,...);

/** @} */

DWRSSDK_END

#endif
