/*--------------------------------------------------------------------------
 *  Copyright (c) 2006-2012 Igor Daniloff.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found in this directory.
 *------------------------------------------------------------------------*/

#ifndef DW_RS_SDK_MAIL_H_
#define DW_RS_SDK_MAIL_H_

#include "DwRsSDK_defs.H"
#include "DwRsSDK_IUnknown.H"

/**\file
   \~russian Поддержка работы с электронными сообщениями.
   \~english Support of e-mail message processing. \~
 */

/** \addtogroup C_INTERFACE
 *  @{
 */

/**\copydoc DwRsSDK::IMail
 */
typedef void* dwrssdk_mail_t;

/**
   \~russian Определяет отправителя в конверте сообщения (envelope).
   \~english Determines the sender in the message envelope. \~

   \param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
   \~english pointer to an object received via ::DwRsSDK_CreateMail. \~

   \param[in] from - \~russian строка с отправителем.
   Если угловые скобки отсутствуют, то они добавляются автоматически.
   \~english a string with the sender.
   If there are no brockets, they will be added automatically. \~

   \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
   \~english 1 - the task was successfully completed, 0 - an error occurred. \~
*/
DWRSSDK_EXTERN int DwRsSDK_SetSender(dwrssdk_mail_t mail,const char* from);
/**\param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
\~english pointer to an object received via ::DwRsSDK_CreateMail. \~

   \copydoc DwRsSDK::IMail::GetSender
*/
DWRSSDK_EXTERN const char* DwRsSDK_GetSender(dwrssdk_mail_t mail);

/**\~russian Добавляет нового получателя в конверт.
   \~english Adds new recipient to the envelope. \~

   \param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
   \~english pointer to an object received via ::DwRsSDK_CreateMail. \~

   \param[in] to - \~russian строка с получателем.
   Если угловые скобки отсутствуют, то они добавляются автоматически.
   \~english a string with the recipient.
   If there are no brockets, they will be added automatically. \~


   \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
   \~english 1 - the task was successfully completed, 0 - an error occurred.  \~
*/
DWRSSDK_EXTERN int DwRsSDK_AddRecipient(dwrssdk_mail_t mail ,const char* to);
/**
   \copydoc DwRsSDK::IMail::GetRecipientsNum

   \param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
   \~english pointer to an object received via ::DwRsSDK_CreateMail. \~
*/
DWRSSDK_EXTERN unsigned int  DwRsSDK_GetRecipientsNum(dwrssdk_mail_t mail);
/**
   \~russian Обеспечивает доступ к определенному получателю из конверта сообщения.
   \~english Provides access to a certain recipient from the message envelope.  \~

   \param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
   \~english pointer to an object received via ::DwRsSDK_CreateMail. \~

   \param[in] pos - \~russian порядковый номер получателя, нумерация получателей начинается с 0.
   \~english index number of the recipient, the recipients enumeration begins with 0. \~

   \return \~russian Строка с получателем (в угловых скобках) или NULL при ошибке.
   \~english A string with the recipient in brockets, or NULL if an error occurs. \~
*/
DWRSSDK_EXTERN const char* DwRsSDK_GetRecipientAt(dwrssdk_mail_t mail,unsigned int pos);

/**\param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
\~english pointer to an object received via ::DwRsSDK_CreateMail. \~

   \copydoc DwRsSDK::IMail::SaveEnvelope
*/
DWRSSDK_EXTERN int DwRsSDK_SaveEnvelope(dwrssdk_mail_t mail);
/**\param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
\~english pointer to an object received via ::DwRsSDK_CreateMail. \~

   \copydoc DwRsSDK::IMail::LoadEnvelope
*/
DWRSSDK_EXTERN int DwRsSDK_LoadEnvelope(dwrssdk_mail_t mail);

/**\param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
\~english pointer to an object received via ::DwRsSDK_CreateMail. \~

   \copydoc DwRsSDK::IMail::GetBodyFilename
*/
DWRSSDK_EXTERN const char* DwRsSDK_GetBodyFilename(dwrssdk_mail_t mail);
/**
   \~russian Сохраняет тело сообщения в его директории.
   \~english Saves the message body in its directory. \~

   \param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
   \~english pointer to an object received via ::DwRsSDK_CreateMail. \~

   \param[in] body - \~russian тело сообщения размера \a size.
   Не обязательно должно заканчиваться NULL.
   \~english message body of \a size. It may or may not end with NULL. \~

   \param[in] size - \~russian размер сообщения.
   \~english message size.  \~

   \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
   \~english 1 - the task was successfully completed, 0 - an error occurred.  \~
*/
DWRSSDK_EXTERN int DwRsSDK_SaveBody(dwrssdk_mail_t mail,const char* body,int size);
/**
   \~russian Загружает тело сообщения из его директории.
   \~english Loads the message body from its directory. \~

   \param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
   \~english Pointer to an object received via ::DwRsSDK_CreateMail. \~

   \param[out] body - \~russian указатель на тело сообщения размера \a size.
   Тело сообщения не обязательно должно заканчиваться NULL.
   Само сообщение сохраняется в объекте IMail и, соответственно, память для \a body
   приложением освобождаться не должна.
   \~english pointer to the message body of \a size. The message itself is stored in
   the IMail object and, respectively, deallocation of the memory for \a body by the
   application is not required. \~

   \param[out] size - \~russian размер сообщения, указатель на который сохранен в \a body.
   \~english message size, the pointer to which is stored in \a body.  \~

   \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
   \~english 1 - the task was successfully completed, 0 - an error occurred.  \~

*/
DWRSSDK_EXTERN int DwRsSDK_LoadBody(dwrssdk_mail_t mail,const char** body,int* size);

/**\param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
\~english Pointer to an object received via ::DwRsSDK_CreateMail. \~

   \copydoc DwRsSDK::IMail::LogEnvelope
*/
DWRSSDK_EXTERN int DwRsSDK_LogEnvelope(dwrssdk_mail_t mail);

/**\param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
\~english Pointer to an object received via ::DwRsSDK_CreateMail. \~

   \copydoc DwRsSDK::IMail::LoadHeaders
*/
DWRSSDK_EXTERN int DwRsSDK_LoadHeaders(dwrssdk_mail_t mail);
/**\param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
\~english pointer to an object received via ::DwRsSDK_CreateMail. \~

   \copydoc DwRsSDK::IMail::GetHeadersNum
*/
DWRSSDK_EXTERN unsigned int DwRsSDK_GetHeadersNum(dwrssdk_mail_t mail);
/**
   \~russian Возвращает заданный заголовок.
   \~english Returns the specified header. \~

   \param[in] mail - \~russian указатель на объект, полученный через ::DwRsSDK_CreateMail.
   \~english pointer to an object received via ::DwRsSDK_CreateMail. \~

   \param[in] pos - \~russian номер заголовка, нумерация начинается с 0.
   \~english number of the header, enumeration begins with 0. \~

   \param[in] name - \~russian название заголовка. Строка заканчивается NULL.
   Память освобождать не нужно.
   \~english name of the header. The string ends with NULL.
   Deallocation of memory is not required. \~

   \param[in] value - \~russian значение заголовка. Строка заканчивается NULL.
   Память освобождать не нужно.
   \~english value of the header. The string ends with NULL.
   Deallocation of memory is not required. \~

   \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
   \~english 1 - the task was successfully completed, 0 - an error occurred. \~
*/
DWRSSDK_EXTERN int DwRsSDK_GetHeaderAt(dwrssdk_mail_t mail,unsigned int pos,const char** name,const char** value);

/**\~russian Создает новый объект dwrssdk_mail_t.
   Значение должно быть освобождено через время, заданное ::DwRsSDK_DeleteMail.
   \~english Creates a new dwrssdk_mail_t object.
   The value must be deallocated in time set by ::DwRsSDK_DeleteMail.\~
   \param[in] dir - \~russian Базовая директория для сообщения. Строка должна заканчиваться знаком '/'.
   \~english Base directory for the message. The string must end with '/' symbol. \~
   \return \~russian Указатель на созданный объект, или NULL при ошибке.
   \~english Pointer to the created object, or NULL if an error occurs. \~
*/
DWRSSDK_EXTERN dwrssdk_mail_t DwRsSDK_CreateMail(const char* dir);
/**\~russian Освобождает память, выделенную через ::DwRsSDK_CreateMail.
\~english Deallocates memory allocated via ::DwRsSDK_CreateMail. \~
   \param[in] m - \~russian указатель на удаляемое сообщение.
   \~english pointer to a message to be deleted. \~
*/
DWRSSDK_EXTERN void           DwRsSDK_DeleteMail(dwrssdk_mail_t m);

/** @} */

DWRSSDK_BEGIN

#ifdef __cplusplus

/**\~russian Объект для хранения и обработки электронных сообщений. Он НЕ является
    потокобезопасным!
   \~english Object for storage and processing of e-mail messages. It is NOT a thread-safe object! \~
 */
class IMail:virtual public IUnknown
{
public:
 /**\~russian Конструктор нового объекта для обработки электронного сообщения
 \~english Constructor of a new object for processing an e-mail message \~
    \param[in] dir - \~russian Базовый каталог для сообщения. Строка должна заканчиваться знаком '/'.
    \~english Base directory for the message. The string must end with '/' symbol. \~
    \sa IProcessingInfo::GetDirToSave
    \sa DwRsSDK_GetDirToSave
  */
 IMail(const char* dir){dir=dir;}
 virtual ~IMail(){}

 /**\~russian Устанавливает отправителя в конверте сообщения (envelope).
 \~english Determines the sender in the message envelope. \~
    \param[in] from - \~russian строка с отправителем.
    Если угловые скобки отсутствуют, то они добавляются автоматически.
    \~english A string with the sender.
    If there are no brockets, they will be added automatically. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
  */
 virtual bool SetSender(const char* from)=0;
 /**\~russian Осуществляет доступ к отправителю из конверта сообщения.
 \~english Provides access to the sender from the message envelope.  \~

    \return \~russian Строка с отправителем, или пустая строка, если отправитель неизвестен.
    \~english A string with the sender or empty string, if the sender is unknown. \~
  */
 virtual const char* GetSender()const=0;

    /**\~russian Добавляет нового получателя в конверт.
       \~english Adds new recipient to the envelope. \~

       \param[in] to - \~russian строка с получателем.
       Если угловые скобки отсутствуют, то они добавляются автоматически.
       \~english a string with the recipient.
       If there are no brockets, they will be added automatically. \~

       \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
    */
 virtual bool AddRecipient(const char* to)=0;
 /**
    \return \~russian Чиcло получателей, заданных в конверте письма.
    \~english Number of recipients specified in the message envelope.  \~
 */
 virtual unsigned int  GetRecipientsNum()const=0;
 /** \~russian Обеспечивает доступ к определенному получателю из конверта сообщения.
 \~english Provides access to a certain recipient from the message envelope.  \~
    \param[in] pos - \~russian порядковый номер получателя, нумерация получателей начинается с 0.
    \~english index number of the recipient, the recipients enumeration begins with 0. \~
    \return \~russian Строка с получателем (в угловых скобках), или NULL при ошибке.
    \~english A string with the recipient in brockets, or NULL if an error occurs. \~
 */
 virtual const char* GetRecipientAt(unsigned int pos)=0;

 /** \~russian Сохраняет информацию из конверта в директории сообщения.
 \~english Saves information from the envelope to the message directory. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
  */
 virtual bool SaveEnvelope()=0;
 /** \~russian Загружает из директории с сообщением информацию из конверта.
 \~english Loads information from the envelope from the message directory. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
  */
 virtual bool LoadEnvelope()=0;
 /** \~russian Возвращает название файла с телом сообщения.
 Приложение может самостоятельно прочитать/создать данный файл, например,
 чтобы не загружать его весь в память, если он достаточно большого размера.
 \~english Returns the name of the file with the message body. The application
 can read from the file or create the file by itself, for example, to avoid loading big files
 to the memory. \~

    \return \~russian Абсолютный путь к файлу, или NULL при ошибке.
    \~english Absolute path to a file, or NULL if an error occurs. \~
  */
 virtual const char* GetBodyFilename()const=0;

 /** \~russian Сохраняет тело сообщения в его директории.
 \~english Saves the message body in its directory. \~
    \param[in] body - \~russian тело сообщения размера \a size.
    Не обязательно должно заканчиваться NULL.
    \~english message body of \a size.
    It may or may not end with NULL. \~
    \param[in] size - \~russian размер сообщения.
    \~english message size.  \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
  */
 virtual bool SaveBody(const char* body,int size)=0;
 /**\~russian Загружает тело сообщения из его директории.
 \~english Loads the message body from its directory. \~
    \param[out] body - \~russian указатель на тело сообщения размера \a size.
    Тело сообщения не обязательно должно заканчиваться NULL. Само сообщение сохраняется в
    объекте IMail и, соответственно, память для \a body приложением освобождаться не должна.
    \~english pointer to the message body of \a size. The message itself is stored in
   the IMail object and, respectively, deallocation of the memory for \a body by the
   application is not required. \~
   \param[out] size - \~russian размер сообщения, указатель на который сохранен в \a body.
    \~english message size the pointer to which is stored in \a body.  \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
  */
 virtual bool LoadBody(const char*& body,int& size)=0;

 /**\~russian Выводит в лог конверта информацию о сообщении.
 \~english Outputs information on the message to the envelope log. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
  */
 virtual bool LogEnvelope()const=0;

 /**\~russian Загружает из каталога сообщения его заголовки.
    Может быть вызвана только после того, как модуль drweb-maild вернул результат
    проверки сообщения, и в результатах установлен флаг DwRsSDK::kACT_Pass.
    \~english Loads message headers from its directory.
    Can be called only after drweb-maild returns the message check result,
    and the DwRsSDK::kACT_Pass flag is set in results. \~


    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
  */
 virtual bool LoadHeaders()=0;
 /**\~russian Возвращает число загруженных полей заголовка.
 \~english Returns the number of loaded fields of a header. \~
    \sa IMail:LoadHeaders
    \return \~russian Число доступных полей заголовка.
    \~english A number of available fields of a header. \~
 */
 virtual unsigned int GetHeadersNum()const=0;
 /**\~russian Возвращает заданный заголовок.
 \~english Returns the specified header. \~
    \param[in] pos - \~russian номер заголовка, нумерация начинается с 0.
    \~english number of the header, enumeration begins with 0. \~
    \param[in] name - \~russian название заголовка. Строка заканчивается NULL. Память освобождать не нужно.
    \~english name of the header. The string ends with NULL. Deallocation of memory is not required. \~
    \param[in] value - \~russian значение заголовка. Строка заканчивается NULL. Память освобождать не нужно.
    \~english value of the header. The string ends with NULL. Deallocation of memory is not required. \~

    \return \~russian 1 - действие успешно выполнено, 0 - произошла ошибка.
    \~english 1 - the task was successfully completed, 0 - an error occurred. \~
  */
 virtual bool GetHeaderAt(unsigned int pos,const char** name,const char** value)const=0;

};

/**\~russian Создание нового объекта IMail.
   Значение должно быть освобождено через DwRsSDK::DeleteMail.
   \~english Creation of a new IMail object.
   The value must be deallocated via DwRsSDK::DeleteMail. \~
   \sa DwRsSDK::DwPtr
   \param[in] dir - IMail::IMail
   \return \~russian Указатель на созданный объект, или NULL при ошибке.
   \~english A pointer to the created object, or NULL if an error occurred. \~
*/
IMail* CreateMail(const char* dir);
/**\~russian Освобождает память, выделенную через DwRsSDK::CreateMail.
\~english Deallocates memory allocated via DwRsSDK::CreateMail. \~
   \param[in] m - \~russian указатель на удаляемое сообщение.
   \~english a pointer to the message to be deleted. \~
*/
void   DeleteMail(IMail* m);

#endif

DWRSSDK_END

#endif
