/*--------------------------------------------------------------------------
 *  Copyright (c) 2006-2012 Igor Daniloff.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found in this directory.
 *------------------------------------------------------------------------*/

#ifndef DW_RS_SDK_DEFS_H_
#define DW_RS_SDK_DEFS_H_

/**\file
   \~russian Общие для всего API макросы, определения и перечисления.
   \~english Macros, definitions and enumerations common for the whole API. \~
 */

#ifdef __cplusplus

/**\namespace DwRsSDK
   \~russian Пространство имен для функций и объектов API.
   \~english Namespaces for API functions and objects. \~
*/

#define DWRSSDK_NAMESPACE DwRsSDK
#define DWRSSDK_BEGIN namespace DWRSSDK_NAMESPACE{
#define DWRSSDK_END }
#define DWRSSDK_USING using namespace  DWRSSDK_NAMESPACE;
#define DWRSSDK_EXTERN extern "C"


#else

#define DWRSSDK_BEGIN
#define DWRSSDK_END
#define DWRSSDK_USING
#define DWRSSDK_EXTERN extern

#endif


/**\~russian Коды возврата из серверной функции обработки запросов.
    \~english Return codes from the request processing function of the server. \~*/
typedef enum{
    PR_StopThread,/*!<\~russian Пришел запрос на остановку данного потока.
    \~english A request to stop the processing of this thread is received. \~*/
    PR_EndProcessing/*!<\~russian Закончена обработка очередного запроса.
    \~english Processing of another request is completed. \~*/
}DwRsSDK_ProcessRes;

/** \~russian Объявление функции обратного вызова для обработки
    запросов в сервисе Callback.
    \~english Declaration of the callback function for processing requests
    in the Callback service.

    \param[in] request - \~russian индекс запроса, переданный в drweb-maild
    \sa DwRsSDK::IProcessingInfo::GetDataToSend, DwRsSDK_GetDataToSend.
    \~english request index passed to drweb-maild \sa DwRsSDK::IProcessingInfo::GetDataToSend,
    DwRsSDK_GetDataToSend. \~

    \param[in] action - \~russian флаги (или их объединение) из ::DwAction, возвращенные
    drweb-maild в ответ на сообщение \sa DwRsSDK::IReceiver::SendRequestToMaild,
    DwRsSDK_SendRequestToMaild.
    \~english flags (or their combination) from ::DwAction, returned by drweb-maild in response
    to the message \sa DwRsSDK::IReceiver::SendRequestToMaild, DwRsSDK_SendRequestToMaild. \~

    \param[in] smtp_answer - \~russian smtp-ответ от drweb-maild (если ответ не был получен,
    указывается пустая строка).
    \~english smtp-response from drweb-maild (if the response was not received, then empty string
    is specified). \~

    \param[in] from - \~russian директория, которую необходимо удалить или переместить
    при успешном приеме запроса. В том случае, если установлен флаг ::kACT_Pass
    и не установлен флаг ::kACT_CanNotDeleteDir - следует выполнить команду типа
    rename(\a from, \a to).
    Если установлен флаг ::kACT_CanNotDeleteDir, то оптимальным решением будет
    скопировать содержимое всей директории \a from в директорию \a to
    \sa DwRsSDK::ICallback::PrepareAfterPassReceived, DwRsSDK_PrepareAfterPassReceived.
    \~english directory, which must be deleted or moved, if the request was successfully received.
    If the kACT_Pass flag is set and the kACT_CanNotDeleteDir flag is not set, then a command
    rename(\a from, \a to) should be executed.
    If the kACT_CanNotDeleteDir flag is set, then it is better to copy the contents of the \a from directory
    to the \a to directory \sa DwRsSDK::ICallback::PrepareAfterPassReceived, DwRsSDK_PrepareAfterPassReceived. \~

    \param[in] to - \~russian директория, в которую должны переместить или скопировать
    директорию \a from при успешном приеме запроса и получении ::kACT_Pass.
    Необходимость такого действия вызвана тем, что если установлен флаг ::kACT_Pass,
    то у компонента может возникнуть необходимость перечитать сообщение, если оно было изменено.
    \~english the directory to which the contents of the \a from directory must be moved or copied,
    if the request and kACT_Pass were successfully received.
    It is necessary to do this because if the the::kACT_Pass flag is set, the component may need
    to re-read the message, if it was modified. \~


    \return \~russian 1 - если данное письмо будет обрабатываться и 0 - в противном случае.
    \~english 1 - if this message is to be processed, otherwise - 0. \~
    */
typedef int (*dwrssdk_callback_processing_t)(const char* request,
                                             int         action,
                                             const char* smtp_answer,
                                             const char* from,
                                             const char* to);

DWRSSDK_BEGIN

typedef DwRsSDK_ProcessRes ProcessRes;

/**\~russian Действия, которые модуль drweb-maild может указать совершить
    над сообщением компонентам Receiver или Sender.
    Тут же - ответы callback-соединения модулю drweb-maild.
   \~english Actions the drweb-maild module may instruct the Receiver and the
    Sender components to perform over the message.
    Here are also responses of the callback-connection to the drweb-maild module. \~*/
typedef enum{
    kACT_Pass                    = 1<<0,/*!<\~russian Сообщение можно пропустить. \~english The message can be passed.  \~*//*= 0x00000001 */
    kACT_Reject                  = 1<<1,/*!<\~russian Заблокировать сообщение и вернуть ошибку (код 5**). \~english Block the message and return an error (code 5**).  \~*//*= 0x00000002 */
    kACT_Tempfail                = 1<<2,/*!<\~russian Временно недоступное сообщение. \~english Temporary unavailable message. \~ *//*= 0x00000004 */
    kACT_Discard                 = 1<<3,/*!<\~russian Заблокировать сообщение и вернуть положительный ответ (код 2**). \~english Block the message and return a positive response (code 2**).  \~*//*= 0x00000008 */

    kACT_None                    = 1<<4,/*!<\~russian Неизвестное действие. \~english Unknown action.  \~ *//*= 0x00000010 */
    kACT_Error                   = 1<<5,/*!<\~russian Ошибка обработки. \~english Processing error.  \~*//*= 0x00000020 */
    kACT_Success                 = 1<<6,/*!<\~russian Сообщение успешно принято. \~english A message is successfully received.  \~*//*= 0x00000040 */

    kACT_ProcessByMyself         = 1<<7,/*!<\~russian Сообщение будет обработано силами самого компонента. \~english A message will be processed by the component.  \~*//*= 0x00000080 */
    kACT_CanNotProcess           = 1<<8,/*!<\~russian Соощение не может быть обработано силами этого компонента. \~english A message cannot be processed by the component.  \~*//*= 0x00000100 */

    kACT_WasBodyModify           = 1<<9,/*!<\~russian Тело сообщения было модифицировано модулем drweb-maild. \~english The message body was modified by the drweb-maild module. \~*//*= 0x00000200 */
    kACT_WasEnvelopeChange       = 1<<10,/*!<\~russian Конверт сообщения (envelope) был изменен модулем drweb-maild. \~english The message envelope was modified by the drweb-maild module.  \~*//*= 0x00000400 */
    kACT_WasTopHeaderModify      = 1<<11,/*!<\~russian Заголовок сообщения был изменен модулем drweb-maild. \~english The message header was modified by the drweb-maild module.  \~*//*= 0x00000800 */
    kACT_WasOnlyAddingNewHeaders = 1<<12,/*!<\~russian Модуль drweb-maild может изменять сообщения только одним способом - добавлять новые заголовки. \~english The drweb-maild module may modify the message only by adding new headers. \~ *//*= 0x00001000 */

    kACT_CanNotDeleteDir         = 1<<13/*!<\~russian Может быть установлен вместе с флагом ::kACT_Pass при возвращении результата сервису Callback, указывая сервису, что исходную директорию требуется сохранить \~english Can be set together with the ::kACT_Pass flag when returning the result to the Callback service, pointing out to the service that the initial directory should be preserved.  \~*//*= 0x00002000 */
} DwAction;

/**\~russian Параметры, передаваемые компоненту Sender.
    \~english Parameters passed to the Sender component.  \~ */
typedef enum{
    kSO_RenameDir = 1<<0,/*!<\~russian  Директория с сообщением полностью переходит под управление компонента Sender. Компонент должен в конце обработки обеспечить удаление этой директории, если обработка прошла успешно (т.е. возвращено kACT_Pass). \~english Message directory passes under the management of Sender component. If message processing was successful (i.e. kACT_Pass is returned), then the component must delete the message directory at the end of processing. \~*/
    kSO_CopyDir   = 1<<1,/*!<\~russian Компонент Sender может только копировать директорию с отправляемым письмом. \~english The Sender component can only copy the directory with the message to be sent. \~*/
    kSO_Sync      = 1<<2/*!<\~russian Ожидается, что Sender будет производить синхронную передачу сообщения (а не складывать сообщение во внутреннюю очередь для дальнейшей обработки, к примеру). \~english It is expected that Sender will perform a synchronous message transfer (it won't put a message to the internal queue for further processing, for example). \~*/
} eSenderOptions;

/**\~russian Параметры, которые передает компонент Receiver модулю drweb-maild при отправке сообщения.
    \~english Parameters passed to the drweb-maild module by the Receiver component during the dispatch of the message.  \~*/
typedef enum{
    kMO_OnlyReceive = 1<<0,/*!<\~russian Компонент Receiver только передает сообщение модулю drweb-maild,
        на этом его функции заканчиваются: компонент ожидает ответ drweb-maild только в плане успешного или
        неуспешного приема сообщения.
        \~english The only function of Receiver component is to pass the message to the drweb-maild module.
        The component waits for response from drweb-maild only to get information about successful or
        unsuccessful receipt of the message.  \~*/
    kMO_ReadyOnlyForBlock = 1<<1,/*!<\~russian Компонент готов сразу получить от drweb-maild и обработать
        ответ о необходимости блокировки сообщения (возможно, временной) и, возможно, SMTP-строку ответа,
        т.е. готов обработать команды kACT_Reject, kACT_Tempfail и kACT_Discard.
        Если drweb-maild не может вернуть ответ (или по результатам проверки письмо надо отправить),
        то drweb-maild должен будет сам отправлять сообщение.
        \~english The component is ready to receive from drweb-maild and immediately process the response about
        message blocking (maybe, temporary) and maybe an SMTP-response string, i.e. it is ready to
        process the kACT_Reject, kACT_Tempfail and kACT_Discard instructions.
        If drweb-maild cannot return the response (or if the message must be sent according to its check results),
        then drweb-maild must send the message itself.  \~*/
    kMO_ReadyForBlockAndSend = 1<<2,/*!<\~russian Компонент готов сразу принять ответ с действием,
        которое необходимо произвести с сообщением (и, возможно, SMTP-строку ответа), либо отправить
        сообщение. При этом, если письмо нужно отправить, допускается его изменение. Параметры изменения
        письма задаются дополнительными флагами.
        \~english The component is ready to receive immediately the response with the action to be performed
        over the message (and maybe an SMTP-response string), or to send the message. At the same time,
        if the message has to be sent, it can be modified. Modification parameters for the message are
        specified by additional flags. \~*/

    kMO_UseBackdoor=1<<9,/*!<\~russian Компонент Receiver позволяет использовать backdoor соединение
        для асинхронных ответов на проверку письма.
    \~english Receiver component allows to use backdoor connection for asynchronous responses to the message check.  \~*/


    kMO_CanSendOnlyWithNewHeaders=1<<10,/*!<\~russian Из всех модификаций письма допускается только добавление новых заголовков.
    \~english Only adding of headers is allowed as modification in the message.  \~*/
    kMO_CanNotEnvelopeModify=1<<11,/*!<\~russian Компонент Receiver не может отсылать письмо, конверт (envelope) которого был модифицирован.
    \~english Receiver component cannot send a message, if its envelope has been modified. \~*/
    kMO_CanNotHeadersModify=1<<12,/*!<\~russian Компонент Receiver не может отсылать письмо, заголовки которого были модифицированы.
    \~english Receiver component cannot send a message, if its headers have been modified.  \~*/
    kMO_CanNotBodyModify=1<<13,/*!<\~russian Компонент Receiver не может отсылать письмо, тело которого было модифицировано.
    \~english Receiver component cannot send a message, if its body has been modified. \~*/
    kMO_ReadOnlyFromMsg=1<<14/*!<\~russian Все изменения в заголовках и в теле сообщения компонент Receiver читает только через оригинальное сообщение.
    \~english All modifications of message body and headers can be read by the Receiver component only via original message.  \~*/
} eMaildOptions;

DWRSSDK_END

#endif
