#!/usr/bin/perl -w
 #######################################################################
 # Copyright 2010 Doctor Web, Ltd.
 # 
 # Following source code is the property of Doctor Web, Ltd.
 # Use is subject to license terms.
 # The reproduction in any form without written permission of
 # Doctor Web, Ltd. and proper attribution is strongly prohibited.
 # 
 # Author: $Author: a.rikov $
 # Date: $Date: 2011/04/28 12:42:05 $
 # Revision: $Revision: 1.55.2.14 $
 #######################################################################
use File::Basename;

use lib "dirname($0)";
use lib "/opt/drweb/maild/scripts";
use lib "/usr/local/drweb/maild/scripts";

use dwconfig;
use Sys::Hostname;
use Cwd;

my @amc_files = qw(
    plugin_modifier.amc
    plugin_vaderetro.amc
    plugin_drweb.amc
    cyrus_sasl.amc
);

my @list_of_possible_web_servers_users = qw(
    apache  www-data
    apache2 httpd
    httpd2  _nginx
    nginx
);

my %licensed_records2plugins = (
   DrWebAntiVirus => 'drweb',
   VRAntiSpam     => 'vaderetro'
);

my @maildesk_langs = qw(en ru);

my $return_code = 0;

sub getKeyRegExp{return IsMailDesk() ? "MailDesk" : "MailD[^aP]";}
sub UseSecureHash{return IsSendmail() || IsQmail();}
sub UseCallbackByDefault{return IsSendmail() || IsQmail() || IsCourier();}
sub UseDrwebReceiver{return IsExim() || IsSmtp() || IsPostfix();}

######################################################################
sub getWebServerUsersName
{
    my @founded_web_servers_users = ();

    my $user;
    foreach $user (@list_of_possible_web_servers_users) {
        next unless getpwnam($user);
        push @founded_web_servers_users, $user;
    }

    return \@founded_web_servers_users;
}

sub addUser2Group
{
    my ($group_name,$user_name) = @_;

    my ($name,$passwd,$gid,$members) = getgrnam($group_name); 

    $members = {map {($_ => 1)} (split /\s+/, $members)};

    if (exists $members->{$user_name} ) {
        printf $lng{allready_in_group}, $user_name, $group_name;
        return 1;
    }

    system('usermod','-G',$group_name,$user_name)==0
        or die (sprintf $lng{'add_user_to_group_err'},$user_name,$group_name);
   
    return 1;
}

######################################################################
sub getTimeZone
{
    @local=(localtime);
    @gmt=(gmtime);
    $diff=$gmt[2]-$local[2];
    if ($gmt[5] > $local[5] || $gmt[7] > $local[7])
    {
        $diff += 24;
    }
    elsif ($gmt[5] < $local[5] || $gmt[7] < $local[7])
    {
        $diff -= 24;
    }

    return ($diff<0?'+':'-').(abs($diff)<10?'0':'').abs($diff).'00';
}

######################################################################
sub getDate
{
    my(undef,undef,undef,$DD,$MM,$YYYY)=(localtime);
    $MM++;
    $YYYY+=1900;
    $DD = '0'.$DD if $DD<10;
    $MM = '0'.$MM if $MM<10; 
    return "$DD-$MM-$YYYY";
}

######################################################################
sub DwGetSecureHash
{
    my $res;
    foreach my $i(1..(int(rand 20)+20)){
        $res.=chr rand(90-65)+65;
    }
    return $res;
}
######################################################################
sub DwGetPlugins
{
    my ($dir) = @_;
    my %files=
        map {/lib(.*)\.so(\.cache)?$/; ($1 => 1)}
            grep /lib(.*)\.so(\.cache)?$/,
                glob($dir."/lib*.so*");

    return keys %files;
}
######################################################################
sub DwGetLngList
{
    my ($dir) = @_;

    my @files=glob($dir."/*.lng");
    my @res;
    my $have_eng=0;
    F: foreach my $file(@files){
        open(FILE, $file) or next F;
        F2: while(<FILE>){
            if(!/(^#)|(^\s*$)/){
                my $new_val = $_;
                $new_val =~ s/\s//g;
                push @res, $new_val unless grep {$_ eq $new_val} @res;
                $have_eng = 1 if $new_val =~ /^en$/;
                last F2;
            }
        }
        close FILE;
    }
    push @res, "en" unless $have_eng;
    return @res;
}
######################################################################
sub WrongEnumName
{
    my ($orig,$new,$msg) = @_;
    NEXT: foreach (@$new){
        foreach my $name(@$orig){
            next NEXT if $name eq $_;
        }
        print sprintf($msg,$_)."\n";
        return 1;
    }
    return 0;
}
######################################################################
sub getKeyFile
{
    my ($dw_bin_dir,$keyfile_ref,$added_src_file_ref,$added_dst_file_ref,$descr) = @_;

    ${$added_dst_file_ref} = $dw_bin_dir.'/drweb32.key';
    my $pattern = $dw_bin_dir.'/drweb32_%s.key';
    my $i=0;
    while(-e ${$added_dst_file_ref}) {
        ${$added_dst_file_ref} = sprintf($pattern, $i);
        $i++;
    }

    my $keyfile = {};
    my $file;
    foreach $file (split(/,/,${$keyfile_ref})) {
        $file =~ s/^\s*//;
        $file =~ s/\s*$//;

        next unless -r $file; # cleaning up from unexistent keys
        $keyfile->{$file} = 1;
    }
    $keyfile->{${$added_dst_file_ref}} = 1;

    my $key_regex = &getKeyRegExp();
    foreach ( keys %{$keyfile} ) {
        if(DwHaveValidKey($_,$key_regex)) {
            ${$keyfile_ref} = ''; 
            return 0;
        }
    }

    KEY:
    my $new_keyfile = '';
    DwEnterString($descr,\$new_keyfile);
    if( $new_keyfile =~ /^\s*$/){
        ${$keyfile_ref}='';
        return 0;
    }elsif(!-e $new_keyfile) {
        printf($lng{'key_not_exists'}."\n",$new_keyfile);
        goto KEY;
    }elsif(!DwHaveValidKey( $new_keyfile, $key_regex ) ){
        printf($lng{'wrong_Key'}."\n",$new_keyfile);
        $new_keyfile = '';
        goto KEY;
    }

    $keyfile = join ", ", keys %{$keyfile};
    ${$added_src_file_ref}=$new_keyfile;
    ${$keyfile_ref} = $keyfile;

    return 1;
}
######################################################################
sub setKeyFile
{
    my ($keyfile_ref,$src_file,$added_file) = @_;

    my $keyfile = {};
    my $file;
    my $modification_flag = 0;
    foreach $file (split(/\s*,\s*/,${$keyfile_ref})) {
        $file =~ s/^\s*//;
        $file =~ s/\s*$//;
        unless(-r $file) {          # claening up from
            $modification_flag = 1; # from nonexistent
            next;                   # keys
        }
        $keyfile->{&Cwd::abs_path($file)} = 1;
    }
    if (!$modification_flag && exists $keyfile->{&Cwd::abs_path($src_file)}) {
        return 0;
    }
    $keyfile->{$added_file} = 1;

    ${$keyfile_ref} = join ", ", keys %{$keyfile};

    return 1; 
}
######################################################################
sub getLicensedPlugins 
{
    my $file = shift;

    my $key_regex = &getKeyRegExp();
    return ()
        if !-r $file || !&DwHaveValidKey($file,$key_regex);

    open(FH, $file)
        or die  $lng{'read_error'}." $file: $!\n";
    my $line;
    while(defined($line = <FH>)) {
        $line =~ s/[\r\n]*$//;
        if ($line =~ /^MailDPlugins\s*=\s*(.*)\s*$/) {
            my $apps = $1;
            return
                split /\s*,\s*/, $apps;
        }
    }
    close(FH);
    return ();
}
######################################################################
sub getInstalledPlugins
{
    my $dw_plugins_dir = shift;

    opendir(DH, $dw_plugins_dir)
        or die  $lng{'read_error'}." '$dw_plugins_dir': $!\n";

    my @installed_plugins = ();
    my $plugin;
    while($plugin = readdir(DH)) {
        next unless -f "$dw_plugins_dir/$plugin";
        $plugin =~ s/^lib(.+?)\.so(\.cache)?$/$1/;
        push @installed_plugins, $plugin; 
    }
    close(DH);

    return @installed_plugins;
}
######################################################################
if (scalar @ARGV ){
    print "$lng{'unrecognize_params'} @ARGV\n";
    exit 1;
}

my $hostname = hostname;
if ($hostname eq "") {
    $hostname = "localhost";
}
if(IsPostfix()){
    $hostname="localhost";
}
my ($dw_root_path,$dw_bin_dir,$dw_etc_dir,$dw_conf_file) = DwInit();
my $dw_etc_dir_def=$dw_etc_dir;
my $enter_dir_flag;
while(1){
    $enter_dir_flag=0; 
    if(! -f "$dw_etc_dir/$dw_conf_file") {
        print "\n".sprintf($lng{'err_conf_dir'},$dw_conf_file,$dw_etc_dir)."\n";
        $enter_dir_flag=1;
    }
    if(! -f "$dw_etc_dir/agent.conf") {
        print "\n".sprintf($lng{'err_conf_dir'},"agent.conf",$dw_etc_dir)."\n";
        $enter_dir_flag=1;
    }
    if ( IsMailDesk() && ! -f "$dw_etc_dir/maildesk-web.conf") {
        print "\n".sprintf($lng{'err_conf_dir'},"maildesk-web.conf",$dw_etc_dir)."\n";
        $enter_dir_flag=1;
    }
    last unless $enter_dir_flag;
    $dw_etc_dir=$dw_etc_dir_def;
    DwEnterString('conf_dir',\$dw_etc_dir);
}
my $drweb_conf      = DwLoadFile("$dw_etc_dir/$dw_conf_file");
my $agent_conf      = DwLoadFile("$dw_etc_dir/agent.conf");
my $maildesk_web_conf;
if ( IsMailDesk() ) {
    $maildesk_web_conf = DwLoadFile("$dw_etc_dir/maildesk-web.conf");
}
my $drweb32ini_conf; 
if(-f "$dw_etc_dir/drweb32.ini") {
    $drweb32ini_conf = DwLoadFile("$dw_etc_dir/drweb32.ini");
}
my $monitor_conf;
if( -f $dw_etc_dir."/monitor.conf" ){
    $monitor_conf = DwLoadFile("$dw_etc_dir/monitor.conf");
}
######################################################################

my $a=GetProductName();
my $FilterMail = "DrWEB-MAIL-DAEMON\@$hostname";
if ( IsMailDesk() ) {
  $a="maild";
  $FilterMail = "DrWeb-MailDesk\@$hostname";
}

my $dw_lng_base_dir = DwGetValue($drweb_conf,"Notifier","LngBaseDir",$dw_etc_dir."/".$a."/lng","string");
$dw_lng_base_dir = $dw_root_path.$dw_lng_base_dir if $dw_lng_base_dir ne $dw_etc_dir."/".$a."/lng";
my $dw_plugins_dir = DwGetValue($drweb_conf,"Filters","LibDir",$dw_bin_dir."/".$a."/plugins","string");
$dw_plugins_dir = $dw_root_path.$dw_plugins_dir if $dw_plugins_dir ne $dw_bin_dir."/".$a."/plugins";
my $Hostname = $hostname;
my $AdminMail;
my $BeforeQueueFilters = DwGetValue($drweb_conf,"Filters","BeforeQueueFilters",'');
my $BeforeQueueFilters_def=$BeforeQueueFilters;
my $AfterQueueFilters = DwGetValue($drweb_conf,"Filters","AfterQueueFilters",'');
my $AfterQueueFilters_def=$AfterQueueFilters;
my $ProtectedNetworks="127.0.0.0/8";
my $ProtectedDomains="$hostname";
my $ReceiverAddress=IsSmtp()?'inet:25@0.0.0.0':'inet:8025@127.0.0.1';
my $SecureHash=DwGetSecureHash();
my $SenderAddress='inet:3003@127.0.0.1' if IsSmtp() && !IsMailDesk();
my $NotifyLangs='en';
my $DaemonKey;
my $ScannerKey;
my $KeyFile =  DwGetValue($agent_conf,"StandaloneMode","LicenseFile","");
my $EmailsFile =  DwGetValue($agent_conf,"StandaloneMode","EmailsFile","","string");
my $KeyFileNew = "";
#my $RunAppList = DwGetValue($monitor_conf,"Monitor","RunAppList","") if $monitor_conf;
my $RunAppList;
if ($monitor_conf) {
    $RunAppList = DwGetValue($monitor_conf,"Monitor","RunAppList","");
    $RunAppList = join ',',
                      grep $_ !~ /^AGENT$/i,
                          split /\s*,\s*/, $RunAppList;
}

if (defined $drweb32ini_conf) {
    $DaemonKey  = DwGetValue($drweb32ini_conf,"Daemon" ,"Key");
    $ScannerKey = DwGetValue($drweb32ini_conf,"Scanner","Key");
}

my $NewKeyFile;
my $NewDaemonKey;
my $NewScannerKey;
my $addedSrcFile;
my $addedDstFile;

################### MailDesk params initialization ##################
my ($daemon_path,       $init_script,
    $ini_file,          $drweb_inject,
    $client_timezone,   $install_date,
    $admin_email,       $admin_password,
    $web_interface_lng, $web_server_user);
if( IsMailDesk() ) {
    $admin_password  = '';
    $client_timezone = getTimeZone();
    $install_date    = getDate();
    $daemon_path     = DwGetValue($maildesk_web_conf,'config','daemon_path' ,'');
    $init_script     = DwGetValue($maildesk_web_conf,'config','init_script' ,'');
    $ini_file        = DwGetValue($maildesk_web_conf,'config','ini_file'    ,'');
    $drweb_inject    = DwGetValue($maildesk_web_conf,'config','drweb_inject','');
}

######################################################################

my $total;
my $enter = sub{
REENTER:

    $NewKeyFile=$KeyFile;
    $NewDaemonKey=$DaemonKey;
    $NewScannerKey=$ScannerKey;

    $addedSrcFile = undef;
    $addedDstFile = undef;

    my $is_key_exists;
    unless ( $is_key_exists = getKeyFile($dw_bin_dir,\$NewKeyFile,\$addedSrcFile,\$addedDstFile,'Key') ) {
        $return_code = '777'
            unless defined $is_key_exists
    }
    if (defined $drweb32ini_conf && length $NewKeyFile) {
        setKeyFile(\$NewDaemonKey,$addedSrcFile,$addedDstFile) or $NewDaemonKey ='';
        setKeyFile(\$NewScannerKey,$addedSrcFile,$addedDstFile) or $NewScannerKey='';
    } else {
        $NewDaemonKey ='';
        $NewScannerKey='';
    }

    #parse key-files to get avialable plugins
    my %plugins_records = ();
    my $file;
    foreach $file (
        split '\s*,\s*', (
            $NewKeyFile
                ? $NewKeyFile.', '.$addedSrcFile
                : $KeyFile
            )
    ) { 
        foreach (&getLicensedPlugins($file)) {
            $plugins_records{$_} = 1;
        }             
    }                 

    my %allowed_plugins = ();
    my $record;           
    foreach $record (keys %plugins_records) {
       next unless exists $licensed_records2plugins{$record};
       $allowed_plugins{ $licensed_records2plugins{$record} } = 1;
    }

    my $plugin;
    foreach $plugin (values %licensed_records2plugins) {
       next if exists $allowed_plugins{$plugin};
       $allowed_plugins{ $plugin } = 0;
    }

    unless( $is_key_exists ) {
        $allowed_plugins{ $_ } = 1
            foreach (getInstalledPlugins($dw_plugins_dir));
    }

#enter all strings
if ( !IsMailDesk ) {
    DwEnterString('plugins_dir',\$dw_plugins_dir)  unless -d $dw_plugins_dir;
    unless(-d $dw_plugins_dir){
        print sprintf($lng{'dir_error'},$dw_plugins_dir)."\n";
        goto REENTER;
    }

    my @plugins_list=DwGetPlugins($dw_plugins_dir);
    @plugins_list = grep(
                            (!exists $allowed_plugins{$_}) || $allowed_plugins{$_},
                            @plugins_list
                        );

    if(scalar @plugins_list){
        my @plugins_list_before;
        foreach $s (@plugins_list) { 
            push(@plugins_list_before, $s) if $s ne "flytrap";
        }

        BEFOREQUEUEFILTERS:
        $BeforeQueueFilters=$BeforeQueueFilters_def;
        DwEnterString('BeforeQueueFilters',\$BeforeQueueFilters,
                      "\n".$lng{'possible_values'}."(".join("|",@plugins_list_before)."). ".$lng{'vector_value'});

        my %before_queue_selected_hash = ();
        if($BeforeQueueFilters){
            my @selected = map {/^\s*(.*?)\s*$/;$1} split m/,/, $BeforeQueueFilters;
            
            goto BEFOREQUEUEFILTERS if WrongEnumName(\@plugins_list_before,\@selected,$lng{'wrong_plugin_name'});

            @plugins_list = grep {
                my $u = 1;
                VAL: foreach my $val( @selected){
                    $u=0,last VAL if $val eq $_;
                }
                $u;
            } @plugins_list;

            my %buffer = %before_queue_selected_hash = map {($_ => 1)} @selected;
            @selected = grep delete $buffer{$_}, @selected;
            $BeforeQueueFilters = join ',', @selected;
        }

        if(@plugins_list){
            AFTERQUEUEFILTERS:
            $AfterQueueFilters=
                join ',',
                    grep !exists $before_queue_selected_hash{$_},
                        split /\s*,\s*/, $AfterQueueFilters_def;
            DwEnterString('AfterQueueFilters',\$AfterQueueFilters,
                          "\n".$lng{'possible_values'}."(".join("|",@plugins_list)."). ".$lng{'vector_value'});
            if($AfterQueueFilters){
                my @selected = map {/^\s*(.*?)\s*$/;$1} split m/,/, $AfterQueueFilters;

                foreach (@selected) {
                    next unless $before_queue_selected_hash{$_};
                    printf $lng{'plugin_already_used'}, $_;
                    print "\n";
                    goto AFTERQUEUEFILTERS;
                }

                goto AFTERQUEUEFILTERS if WrongEnumName(\@plugins_list,\@selected,$lng{'wrong_plugin_name'});

                my %selected_hash = map {($_ => 1)} @selected;
                @selected = grep delete $selected_hash{$_}, @selected;
                $AfterQueueFilters = join ',', @selected;
            }
        }
    }
}
    DwEnterString('Hostname',\$Hostname) if IsSmtp();
    $AdminMail =  "postmaster\@$Hostname";
    if ( IsMailDesk() ) {
	$FilterMail = "DrWeb-MailDesk\@$Hostname";
    } else {
	$FilterMail = "DrWEB-MAIL-DAEMON\@$Hostname";
    }
    DwEnterString('AdminMail',\$AdminMail);
    DwEnterString('FilterMail',\$FilterMail);
    DwEnterString('ProtectedNetworks',\$ProtectedNetworks," ".$lng{'vector_value'}) unless IsMailDesk();
    $ProtectedDomains="$Hostname";
    DwEnterString('ProtectedDomains',\$ProtectedDomains," ".$lng{'vector_value'}) unless IsMailDesk();
    DwEnterString('ReceiverAddress',\$ReceiverAddress," ".$lng{'vector_value'}) if IsSmtp() && !IsMailDesk();
    DwEnterString('SenderAddress',\$SenderAddress) if IsSmtp() && !IsMailDesk();
    DwEnterString('SecureHash',\$SecureHash) if UseSecureHash();

    my $first_password_variant;
    if ( IsMailDesk() ) {
        $admin_email = $AdminMail;

WEB_SERVER_USER:
        my $web_servers_users = getWebServerUsersName();
        $web_server_user = '';
        DwEnterString('web_server_user',
                      \$web_server_user,
                      (
                          scalar @$web_servers_users
                          ? "\n".$lng{'founded_webserver_users'}."(".join("|",@$web_servers_users).")"
                          : ''
                      )
                  );
        if($web_server_user =~ /^\s*$/) {
            print "\n",$lng{'empty_web_server_user_warn'},"\n";
        } elsif (!getpwnam($web_server_user)) {
            printf $lng{'uncknown_user_warn'},$web_server_user;
            goto WEB_SERVER_USER;
        }

        my $fn;
        foreach ((['daemon_path'   ,\$daemon_path ],
                  ['init_script'   ,\$init_script ],
                  ['ini_file'      ,\$ini_file    ],
                  ['drweb_inject'  ,\$drweb_inject])) {
            $fn = ${$_->[1]};
            while(!defined $fn || $fn =~ /^\s*$/ || ! -e $fn) {
                DwEnterString($_->[0],$_->[1] );
                $fn = ${$_->[1]};
                if (!defined $fn || $fn =~ /^\s*$/) {
                    ${$_->[1]} = '';
                    print $lng{'file_name_err'},"\n";
                } elsif (! -f $fn) {
                    printf $lng{'file_exist_err'},$fn,"\n";
                }
            }
        }

        while(1) {
            DwEnterPsswd ('admin_password',\$admin_password);
            $first_password_variant = $admin_password;
            if ($first_password_variant eq '') {
                print $lng{'empty_password'},"\n";
            } else {
                $admin_password = '';
                DwEnterPsswd ($lng{'repeat'},\$admin_password);
                if ($first_password_variant ne $admin_password) {
                    print $lng{'unequal_password'},"\n";
                } else {
                    last;
                }
            }
        }
        eval {
            require Digest::MD5;
            import Digest::MD5 'md5_hex';
        };
        if ($@) {
            require Digest::Perl::MD5;
            import Digest::Perl::MD5 'md5_hex';
        }
        $admin_password = md5_hex($admin_password);
    } 

DW_LNG_BASE_DIR:
    DwEnterString('lng_dir',\$dw_lng_base_dir)  unless -d $dw_lng_base_dir;
    unless(-d $dw_lng_base_dir){
        print sprintf($lng{'dir_error'},$dw_lng_base_dir)."\n";
        goto DW_LNG_BASE_DIR;
    }
    my @lng_list=IsMailDesk() ? @maildesk_langs : DwGetLngList($dw_lng_base_dir);
    my $notify_lng = $NotifyLangs;
NOTIFYLANGS:
    $NotifyLangs = $notify_lng;
    DwEnterString('NotifyLangs',\$NotifyLangs,
                  "\n".$lng{'possible_values'}."(".join("|",@lng_list)."). ".$lng{'vector_value'});
    my @selected = map {/^\s*(.*?)\s*$/;$1} split m/,/, $NotifyLangs;
    goto NOTIFYLANGS if WrongEnumName(\@lng_list,\@selected,$lng{'wrong_lng_name'});

    if(IsMailDesk()) {
        my %lang_hash = map {($_=>1)} @maildesk_langs;
WEB_INTERFACE_LANG:
        $web_interface_lng = $maildesk_langs[0];
        DwEnterString('web_interface_lng',\$web_interface_lng,
                      "\n".$lng{'possible_values'}."(".join("|",@maildesk_langs)."). ");
        unless (exists $lang_hash{$web_interface_lng}) {
            printf $lng{'wrong_lng_name'},"\n";
            goto WEB_INTERFACE_LANG;
        }
    }

    $total = [
        $lng{'plugins_dir_l'}." = ".$dw_plugins_dir,
        $lng{'lng_dir_l'}." = ".$dw_lng_base_dir,
        $lng{'BeforeQueueFilters_l'}." = ".$BeforeQueueFilters,
        $lng{'AfterQueueFilters_l'}." = ".$AfterQueueFilters,
        IsSmtp() ? $lng{'Hostname_l'}." = ".$Hostname : (),
        $lng{'AdminMail_l'}." = ".$AdminMail,
        $lng{'FilterMail_l'}." = ".$FilterMail,
        IsMailDesk() ? () : $lng{'ProtectedNetworks_l'}." = ".$ProtectedNetworks,
        IsMailDesk() ? () : $lng{'ProtectedDomains_l'}." = ".$ProtectedDomains,
        IsSmtp() ? $lng{'ReceiverAddress_l'}." = ".$ReceiverAddress : (),
        IsSmtp() && !IsMailDesk() ? $lng{'SenderAddress_l'}." = ".$SenderAddress : (),
        UseSecureHash() ? $lng{'SecureHash_l'}." = ".$SecureHash : (),
        $lng{'NotifyLangs_l'}." = ".$NotifyLangs,
        length $NewKeyFile    ? $lng{'Key_l'}." = ".$NewKeyFile : (),
        length $NewDaemonKey  ? $lng{'DaemonKey_l' }." = ".$NewDaemonKey : (),
        length $NewScannerKey ? $lng{'ScannerKey_l'}." = ".$NewScannerKey : (),
    ];

    if(IsMailDesk()) {
        push @$total, (
            "", #setting delimeter
            $lng{'daemon_path_l'      }.' = '.$daemon_path,
            $lng{'ini_file_l'         }.' = '.$ini_file,
            $lng{'init_script_l'      }.' = '.$init_script,
            $lng{'drweb_inject_l'     }.' = '.$drweb_inject,
            $lng{'admin_password_l'   }.' = '.('*' x length($first_password_variant)),
            $lng{'admin_email_l'      }.' = '.$admin_email,
            $lng{'web_interface_lng_l'}.' = '.$web_interface_lng,
            $lng{'web_server_user_l'  }.' = '.($web_server_user =~ /\S/ ? $web_server_user : $lng{'web_server_empty_val'}),
        );
    }

};

DwEnterConfig($enter,\$total);

######################################################################

$drweb_conf = DwModify($drweb_conf,"General","Hostname",$Hostname,"string");#ask only for smtp
$drweb_conf = DwModify($drweb_conf,"Notifier","AdminMail",$AdminMail,"string");
$drweb_conf = DwModify($drweb_conf,"Maild","RedirectMail",$AdminMail,"string");#AdminMail==RedirectMail by default
$drweb_conf = DwModify($drweb_conf,"Notifier","FilterMail",$FilterMail,"string");
$drweb_conf = DwModify($drweb_conf,"Filters","AfterQueueFilters",$AfterQueueFilters,"vector");
$drweb_conf = DwModify($drweb_conf,"Filters","BeforeQueueFilters",$BeforeQueueFilters,"vector");
$drweb_conf = DwModify($drweb_conf,"Receiver","Address",$ReceiverAddress,"vector")  if IsSmtp();
$drweb_conf = DwModify($drweb_conf,"Maild","ProtectedNetworks",$ProtectedNetworks,"vector") unless IsMailDesk();
$drweb_conf = DwModify($drweb_conf,"Maild","ProtectedDomains",$ProtectedDomains,"vector") unless IsMailDesk();
$drweb_conf = DwModify($drweb_conf,"Sender","SecureHash",$SecureHash,"string") if UseSecureHash();
$drweb_conf = DwModify($drweb_conf,"Sender","Address",$SenderAddress,"vector")  if IsSmtp() && !IsMailDesk();
$drweb_conf = DwModify($drweb_conf,"Notifier","NotifyLangs",$NotifyLangs,"vector");

if (IsMailDesk()) {
    $maildesk_web_conf = DwModify($maildesk_web_conf,'config','daemon_path',$daemon_path,'string');
    $maildesk_web_conf = DwModify($maildesk_web_conf,'config','ini_file',$ini_file,'string');
    $maildesk_web_conf = DwModify($maildesk_web_conf,'config','init_script',$init_script,'string');
    $maildesk_web_conf = DwModify($maildesk_web_conf,'config','drweb_inject',$drweb_inject,'string');
    $maildesk_web_conf = DwModify($maildesk_web_conf,'config','admin_email',$admin_email,'string');
    $maildesk_web_conf = DwModify($maildesk_web_conf,'config','lang',$web_interface_lng,'string');
    $maildesk_web_conf = DwModify($maildesk_web_conf,'config','daemon_path',$daemon_path,'string');
    $maildesk_web_conf = DwModify($maildesk_web_conf,'config','admin_password',$admin_password,'string');
    $maildesk_web_conf = DwModify($maildesk_web_conf,'config','client_timezone',$client_timezone,'string');
    $maildesk_web_conf = DwModify($maildesk_web_conf,'config','install_date',$install_date,'string');
    DwSaveConf("$dw_etc_dir/maildesk-web.conf",$maildesk_web_conf);

    my $file;
    foreach $file (@amc_files) {
        &DwPatchAmcFileParam("$dw_bin_dir/agent/$file",'Application','"MAILD"','"MAILDESK"');
    }

    addUser2Group('drweb',$web_server_user) if $web_server_user !~ /^\s*$/;
}

if(length $NewKeyFile){
    DwCopyFile($addedSrcFile,$addedDstFile);
    print "\n".sprintf($lng{'install_key'},$addedSrcFile, $addedDstFile)."\n";

    $agent_conf = DwModify($agent_conf,"StandaloneMode","LicenseFile",$NewKeyFile,"vector");
    DwSaveConf("$dw_etc_dir/agent.conf",$agent_conf);
}

if (length $NewDaemonKey || length $NewScannerKey) {
        $drweb32ini_conf = DwModify($drweb32ini_conf,"Daemon" ,"Key",$NewDaemonKey,"multi_string" ) if length $NewDaemonKey;
        $drweb32ini_conf = DwModify($drweb32ini_conf,"Scanner","Key",$NewScannerKey,"multi_string") if length $NewScannerKey;
        DwSaveConf("$dw_etc_dir/drweb32.ini",$drweb32ini_conf);
}

my $product_name_in_monitor = GetProductMonitorName();

if(-f $dw_etc_dir."/monitor.conf" && $RunAppList !~ /\b$product_name_in_monitor\b/){
    my $res = $RunAppList;
    if ( $product_name_in_monitor eq "MAILDESK" && $RunAppList =~ /\bMAILD\b/ ) {
	$res =~ s/\bMAILD\b/$product_name_in_monitor/;
    } elsif ($product_name_in_monitor eq "MAILD" && $RunAppList =~ /\bMAILDESK\b/) {
        $res =~ s/\bMAILDESK\b/$product_name_in_monitor/;
    } else {
	$res.=", " if $res =~ /\S/;
	$res.="$product_name_in_monitor";
    }
    $monitor_conf = DwModify($monitor_conf,"Monitor","RunAppList",$res,"vector");
    DwSaveConf("$dw_etc_dir/monitor.conf",$monitor_conf);
}

DwSaveConf("$dw_etc_dir/$dw_conf_file",$drweb_conf);

exit $return_code;
