#!/usr/bin/perl -w

 #######################################################################
 # Copyright 2007 Igor Daniloff.
 # 
 # Following source code is the property of Doctor Web, Ltd.
 # Use is subject to license terms.
 # The reproduction in any form without written permission of
 # Doctor Web, Ltd. and proper attribution is strongly prohibited.
 #######################################################################

package dwplugins;
use Exporter;
use dwconfig;

@ISA     = qw(Exporter);

@EXPORT = qw(DwPluginInstallationFramework AddPlugin RemovePlugin);
$VERSION = 1.00;
######################################################################
sub DwGet
{
    my ($drweb_conf) = @_;
    my @before = map {/^\s*(.*?)\s*$/} split(/,/,DwGetValue($drweb_conf,"Filters","BeforeQueueFilters",""));
    my @after  = map {/^\s*(.*?)\s*$/} split(/,/,DwGetValue($drweb_conf,"Filters","AfterQueueFilters",""));
    return (\@before,\@after);
}

######################################################################
sub RemovePlugin
{
    my ($drweb_conf,$name) = @_;
    my ($b,$a) = DwGet($drweb_conf);

    my @before_new = grep{$_ ne $name} @$b;
    my @after_new  = grep{$_ ne $name} @$a;

    $drweb_conf = DwModify($drweb_conf,"Filters","BeforeQueueFilters",join(", ",@before_new),"vector") if @before_new != @$b;
    $drweb_conf = DwModify($drweb_conf,"Filters","AfterQueueFilters",join(", ",@after_new),"vector") if @after_new != @$a;
 
    return $drweb_conf;
}

######################################################################
sub AddPlugin
{
    my ($drweb_conf,$name,$can_before_queue,$change_list_ref) = @_;
    my ($before1,$after1) = DwGet($drweb_conf);
    my @before = @$before1;
    my @after  = @$after1;

    foreach (@before, @after){
        if ( $_ eq $name ){
            print sprintf "$lng{'pc_patched'}\n", $name;
            return $drweb_conf;
        }
    }
    my $add="y";
    if(!$can_before_queue || $can_before_queue ne "no"){
        DwEnterString('pc_before',\$add);
    }else{
        $add="n";
    }
    my $queue_name;
    my @queue;
    if(lc($add) eq "y"){
        $queue_name = "BeforeQueueFilters";
        @queue = @before;
    }else{
        $add="y";
        DwEnterString('pc_after',\$add);
        if( lc($add) ne "y" ) {
            return $drweb_conf;
        }
        $queue_name = "AfterQueueFilters";
        @queue = @after;
    }
    if(!scalar @queue){
        push @queue, $name;
        push @${$change_list_ref}, sprintf($lng{'AddedPlugin_l'}, $queue_name, $name);
    }else{
        my $pos = 0;
      QUESTION:
        print sprintf("\n\n$lng{'pc_add'}\n",$queue_name);
        print sprintf("$lng{'pc_question_b'}",$name,$queue[$pos]) 
            if $pos<scalar @queue;
        print sprintf("$lng{'pc_question_a'}",$name,$queue[$pos-1]) 
            unless $pos<scalar @queue;
        print "$lng{'pc_question_l'}" if $pos>0;
        print "$lng{'pc_question_r'}" if $pos<scalar @queue;
        print "$lng{'pc_question_e'}\n";
        my $confirm = <>; chomp $confirm; 
        return $drweb_conf if $confirm eq "7";
        if($confirm eq "1"){
            splice @queue, $pos,0,$name;
            push @${$change_list_ref}, sprintf($lng{'AddedPlugin_l'}, $queue_name, join(",", @queue));
        }else{
            if($confirm eq "3" && $pos>0){
                --$pos;
            }elsif($confirm eq "5" && $pos<scalar @queue){
                ++$pos;
            }else{
                print $lng{'invalid_option'}."\n";
            }
            goto QUESTION;
        }
    }
    if ( $queue_name eq "AfterQueueFilters" ){
        my $b = DwGetValue($drweb_conf,"Maild","InMaxThreads","");
        my $a = DwGetValue($drweb_conf,"Maild","OutMaxThreads","");
        if($a eq "1" && $b ne ""){
            $drweb_conf = DwModify($drweb_conf,"Maild","OutMaxThreads",$b,"int");
        }
    }

    return DwModify($drweb_conf,"Filters",$queue_name,join(", ",@queue),"vector");
}

######################################################################
sub DwPluginInstallationFramework
{
    if (scalar @ARGV ){
        print "$lng{'unrecognize_params'} @ARGV\n";
        return;
    }
    
    my $need_restart = 1;

    my ($name,$specific,$can_before_queue,$name_in_key) = @_;
    my ($dw_root_path,$dw_bin_dir,$dw_etc_dir,$dw_conf_file) = DwInit();
    my $dw_monitor_init = DwInitInit();

    my $dw_etc_dir_def=$dw_etc_dir;
    while(! -f "$dw_etc_dir/$dw_conf_file" || ! -f "$dw_etc_dir/plugin_$name.conf"){
        print "\n".sprintf($lng{'err_conf_dir'},$dw_conf_file,$dw_etc_dir)."\n" 
            if ! -f "$dw_etc_dir/$dw_conf_file";
        print "\n".sprintf($lng{'err_conf_dir'},"plugin_$name.conf",$dw_etc_dir)."\n" 
            unless ! -f "$dw_etc_dir/$dw_conf_file";
        $dw_etc_dir=$dw_etc_dir_def;
        DwEnterString('conf_dir',\$dw_etc_dir);
    }

    $drweb_conf = DwLoadFile("$dw_etc_dir/$dw_conf_file");

    if( $name_in_key && -f "$dw_etc_dir/agent.conf"){
        my $agent_conf = DwLoadFile("$dw_etc_dir/agent.conf");
        my $KeyFile =  DwGetValue($agent_conf,"StandaloneMode","LicenseFile","");
        $KeyFile = join (", ", grep { -r $_ } map{/^\s*(.*)\s*$/} split(",",$KeyFile));
        my $NewKeyFile="";
        if(!DwHaveValidKey($KeyFile,$name_in_key)){
          KEY:
            DwEnterString(sprintf($lng{'PluginKey'}."\n",$name),\$NewKeyFile);
            unless($NewKeyFile){
                $need_restart = 0;
            }elsif(!DwHaveValidKey($NewKeyFile,$name_in_key)){
                print sprintf($lng{'wrong_plugin_Key'}."\n",$NewKeyFile,$name);
                #goto KEY;
            }else{
                $KeyFile.=", " if $KeyFile =~ /\S/;
                $KeyFile.=$NewKeyFile;
                $agent_conf = DwModify($agent_conf,"StandaloneMode","LicenseFile",$KeyFile,"vector");
                DwSaveConf("$dw_etc_dir/agent.conf",$agent_conf);
            }
        }
    }
    
    my $old_drweb_conf=$drweb_conf;
    if( $specific ){
        my $plugin_conf = DwLoadFile("$dw_etc_dir/plugin_$name.conf");
        my $old_plugin_conf=$plugin_conf;
        ($drweb_conf,$plugin_conf) = $specific->($drweb_conf,$old_drweb_conf,$plugin_conf,\$change_list);
        if($old_plugin_conf ne $plugin_conf){
            system("cp $dw_etc_dir/plugin_$name.conf $dw_etc_dir/plugin_$name.conf.drwebsave")==0
                or die ((sprintf $lng{'copy_error'},"dw_etc_dir/plugin_$name.conf","$dw_etc_dir/plugin_$name.conf").": ".$!);
            DwSaveFile("$dw_etc_dir/plugin_$name.conf",$plugin_conf);
            print "\n$dw_etc_dir/plugin_$name.conf ".$lng{'ok_patch'}."\n\n";
        }
   } else {
       my $change_list;
       my $enter = sub {
           $change_list = [];
           $drweb_conf = AddPlugin($old_drweb_conf,$name,$can_before_queue,\$change_list);
       };
       DwEnterConfig($enter,\$change_list); 
   }
    
    if($old_drweb_conf ne $drweb_conf){
        if( ! -f "$dw_etc_dir/$dw_conf_file.drwebsave"){
            system("cp $dw_etc_dir/$dw_conf_file $dw_etc_dir/$dw_conf_file.drwebsave")==0
                or die ((sprintf $lng{'copy_error'},"$dw_etc_dir/$dw_conf_file","$dw_etc_dir/$dw_conf_file.drwebsave").": ".$!);
        }
        DwSaveFile("$dw_etc_dir/$dw_conf_file",$drweb_conf);
        print "\n$dw_etc_dir/$dw_conf_file ".$lng{'ok_patch'}."\n\n";
    }
   
    if($need_restart){
	if (system("$dw_monitor_init", "restart") == 0) {
	    print "\n".$lng{'monitor_restart'}."\n\n";
	}
    }
}
